from .const import SUPPORT_ALARM_ARM_AWAY as SUPPORT_ALARM_ARM_AWAY, SUPPORT_ALARM_ARM_CUSTOM_BYPASS as SUPPORT_ALARM_ARM_CUSTOM_BYPASS, SUPPORT_ALARM_ARM_HOME as SUPPORT_ALARM_ARM_HOME, SUPPORT_ALARM_ARM_NIGHT as SUPPORT_ALARM_ARM_NIGHT, SUPPORT_ALARM_ARM_VACATION as SUPPORT_ALARM_ARM_VACATION, SUPPORT_ALARM_TRIGGER as SUPPORT_ALARM_TRIGGER
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_CODE as ATTR_CODE, ATTR_CODE_FORMAT as ATTR_CODE_FORMAT, SERVICE_ALARM_ARM_AWAY as SERVICE_ALARM_ARM_AWAY, SERVICE_ALARM_ARM_CUSTOM_BYPASS as SERVICE_ALARM_ARM_CUSTOM_BYPASS, SERVICE_ALARM_ARM_HOME as SERVICE_ALARM_ARM_HOME, SERVICE_ALARM_ARM_NIGHT as SERVICE_ALARM_ARM_NIGHT, SERVICE_ALARM_ARM_VACATION as SERVICE_ALARM_ARM_VACATION, SERVICE_ALARM_DISARM as SERVICE_ALARM_DISARM, SERVICE_ALARM_TRIGGER as SERVICE_ALARM_TRIGGER
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.config_validation import make_entity_service_schema as make_entity_service_schema
from homeassistant.helpers.entity import Entity as Entity, EntityDescription as EntityDescription
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any, Final

_LOGGER: Final[Any]
DOMAIN: Final[str]
SCAN_INTERVAL: Final[Any]
ATTR_CHANGED_BY: Final[str]
FORMAT_TEXT: Final[str]
FORMAT_NUMBER: Final[str]
ATTR_CODE_ARM_REQUIRED: Final[str]
ENTITY_ID_FORMAT: Final[Any]
ALARM_SERVICE_SCHEMA: Final[Any]
PLATFORM_SCHEMA: Final[Any]
PLATFORM_SCHEMA_BASE: Final[Any]

async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class AlarmControlPanelEntityDescription(EntityDescription): ...

class AlarmControlPanelEntity(Entity):
    entity_description: AlarmControlPanelEntityDescription
    _attr_changed_by: Union[str, None]
    _attr_code_arm_required: bool
    _attr_code_format: Union[str, None]
    _attr_supported_features: int
    @property
    def code_format(self) -> Union[str, None]: ...
    @property
    def changed_by(self) -> Union[str, None]: ...
    @property
    def code_arm_required(self) -> bool: ...
    def alarm_disarm(self, code: Union[str, None] = ...) -> None: ...
    async def async_alarm_disarm(self, code: Union[str, None] = ...) -> None: ...
    def alarm_arm_home(self, code: Union[str, None] = ...) -> None: ...
    async def async_alarm_arm_home(self, code: Union[str, None] = ...) -> None: ...
    def alarm_arm_away(self, code: Union[str, None] = ...) -> None: ...
    async def async_alarm_arm_away(self, code: Union[str, None] = ...) -> None: ...
    def alarm_arm_night(self, code: Union[str, None] = ...) -> None: ...
    async def async_alarm_arm_night(self, code: Union[str, None] = ...) -> None: ...
    def alarm_arm_vacation(self, code: Union[str, None] = ...) -> None: ...
    async def async_alarm_arm_vacation(self, code: Union[str, None] = ...) -> None: ...
    def alarm_trigger(self, code: Union[str, None] = ...) -> None: ...
    async def async_alarm_trigger(self, code: Union[str, None] = ...) -> None: ...
    def alarm_arm_custom_bypass(self, code: Union[str, None] = ...) -> None: ...
    async def async_alarm_arm_custom_bypass(self, code: Union[str, None] = ...) -> None: ...
    @property
    def supported_features(self) -> int: ...
    @property
    def state_attributes(self) -> Union[dict[str, Any], None]: ...

class AlarmControlPanel(AlarmControlPanelEntity):
    def __init_subclass__(cls, **kwargs: Any) -> None: ...



import asyncio
from .addon import AddonError as AddonError, AddonManager as AddonManager, AddonState as AddonState, get_addon_manager as get_addon_manager
from .api import async_register_api as async_register_api
from .const import ATTR_COMMAND_CLASS as ATTR_COMMAND_CLASS, ATTR_COMMAND_CLASS_NAME as ATTR_COMMAND_CLASS_NAME, ATTR_DATA_TYPE as ATTR_DATA_TYPE, ATTR_ENDPOINT as ATTR_ENDPOINT, ATTR_EVENT as ATTR_EVENT, ATTR_EVENT_DATA as ATTR_EVENT_DATA, ATTR_EVENT_LABEL as ATTR_EVENT_LABEL, ATTR_EVENT_TYPE as ATTR_EVENT_TYPE, ATTR_HOME_ID as ATTR_HOME_ID, ATTR_LABEL as ATTR_LABEL, ATTR_NODE_ID as ATTR_NODE_ID, ATTR_PARAMETERS as ATTR_PARAMETERS, ATTR_PROPERTY as ATTR_PROPERTY, ATTR_PROPERTY_KEY as ATTR_PROPERTY_KEY, ATTR_PROPERTY_KEY_NAME as ATTR_PROPERTY_KEY_NAME, ATTR_PROPERTY_NAME as ATTR_PROPERTY_NAME, ATTR_TYPE as ATTR_TYPE, ATTR_VALUE as ATTR_VALUE, ATTR_VALUE_RAW as ATTR_VALUE_RAW, CONF_ADDON_DEVICE as CONF_ADDON_DEVICE, CONF_ADDON_NETWORK_KEY as CONF_ADDON_NETWORK_KEY, CONF_DATA_COLLECTION_OPTED_IN as CONF_DATA_COLLECTION_OPTED_IN, CONF_INTEGRATION_CREATED_ADDON as CONF_INTEGRATION_CREATED_ADDON, CONF_NETWORK_KEY as CONF_NETWORK_KEY, CONF_USB_PATH as CONF_USB_PATH, CONF_USE_ADDON as CONF_USE_ADDON, DATA_CLIENT as DATA_CLIENT, DATA_PLATFORM_SETUP as DATA_PLATFORM_SETUP, DOMAIN as DOMAIN, EVENT_DEVICE_ADDED_TO_REGISTRY as EVENT_DEVICE_ADDED_TO_REGISTRY, LOGGER as LOGGER, ZWAVE_JS_NOTIFICATION_EVENT as ZWAVE_JS_NOTIFICATION_EVENT, ZWAVE_JS_VALUE_NOTIFICATION_EVENT as ZWAVE_JS_VALUE_NOTIFICATION_EVENT, ZWAVE_JS_VALUE_UPDATED_EVENT as ZWAVE_JS_VALUE_UPDATED_EVENT
from .discovery import ZwaveDiscoveryInfo as ZwaveDiscoveryInfo, async_discover_values as async_discover_values
from .helpers import async_enable_statistics as async_enable_statistics, get_device_id as get_device_id, get_unique_id as get_unique_id
from .migrate import async_migrate_discovered_value as async_migrate_discovered_value
from .services import ZWaveServices as ZWaveServices
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_DEVICE_ID as ATTR_DEVICE_ID, ATTR_DOMAIN as ATTR_DOMAIN, ATTR_ENTITY_ID as ATTR_ENTITY_ID, CONF_URL as CONF_URL, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import device_registry as device_registry, entity_registry as entity_registry
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from homeassistant.helpers.dispatcher import async_dispatcher_send as async_dispatcher_send
from zwave_js_server.client import Client as ZwaveClient
from zwave_js_server.model.node import Node as ZwaveNode
from zwave_js_server.model.notification import NotificationNotification as NotificationNotification
from zwave_js_server.model.value import Value as Value, ValueNotification as ValueNotification

CONNECT_TIMEOUT: int
DATA_CLIENT_LISTEN_TASK: str
DATA_START_PLATFORM_TASK: str
DATA_CONNECT_FAILED_LOGGED: str
DATA_INVALID_SERVER_VERSION_LOGGED: str

async def async_setup(hass: HomeAssistant, config: dict) -> bool: ...
def register_node_in_dev_reg(hass: HomeAssistant, entry: ConfigEntry, dev_reg: device_registry.DeviceRegistry, client: ZwaveClient, node: ZwaveNode) -> device_registry.DeviceEntry: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def client_listen(hass: HomeAssistant, entry: ConfigEntry, client: ZwaveClient, driver_ready: asyncio.Event) -> None: ...
async def disconnect_client(hass: HomeAssistant, entry: ConfigEntry) -> None: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_remove_entry(hass: HomeAssistant, entry: ConfigEntry) -> None: ...
async def async_ensure_addon_running(hass: HomeAssistant, entry: ConfigEntry) -> None: ...
def async_ensure_addon_updated(hass: HomeAssistant) -> None: ...



from homeassistant import config_entries as config_entries
from homeassistant.components.pi_hole.const import CONF_LOCATION as CONF_LOCATION, CONF_STATISTICS_ONLY as CONF_STATISTICS_ONLY, DEFAULT_LOCATION as DEFAULT_LOCATION, DEFAULT_NAME as DEFAULT_NAME, DEFAULT_SSL as DEFAULT_SSL, DEFAULT_STATISTICS_ONLY as DEFAULT_STATISTICS_ONLY, DEFAULT_VERIFY_SSL as DEFAULT_VERIFY_SSL, DOMAIN as DOMAIN
from homeassistant.const import CONF_API_KEY as CONF_API_KEY, CONF_HOST as CONF_HOST, CONF_NAME as CONF_NAME, CONF_PORT as CONF_PORT, CONF_SSL as CONF_SSL, CONF_VERIFY_SSL as CONF_VERIFY_SSL
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from typing import Any

_LOGGER: Any

class PiHoleFlowHandler(config_entries.ConfigFlow):
    VERSION: int
    _config: Any
    def __init__(self) -> None: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_import(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_init(self, user_input: Union[dict[str, Any], None], is_import: bool = ...) -> FlowResult: ...
    async def async_step_api_key(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def _async_endpoint_existed(self, endpoint: str) -> bool: ...
    async def _async_try_connect(self, host: str, location: str, tls: bool, verify_tls: bool) -> None: ...



from .const import DATA_COORDINATOR as DATA_COORDINATOR, DOMAIN as DOMAIN
from .coordinator import CanaryDataUpdateCoordinator as CanaryDataUpdateCoordinator
from canary.api import Location as Location
from homeassistant.components.alarm_control_panel import AlarmControlPanelEntity as AlarmControlPanelEntity
from homeassistant.components.alarm_control_panel.const import SUPPORT_ALARM_ARM_AWAY as SUPPORT_ALARM_ARM_AWAY, SUPPORT_ALARM_ARM_HOME as SUPPORT_ALARM_ARM_HOME, SUPPORT_ALARM_ARM_NIGHT as SUPPORT_ALARM_ARM_NIGHT
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import STATE_ALARM_ARMED_AWAY as STATE_ALARM_ARMED_AWAY, STATE_ALARM_ARMED_HOME as STATE_ALARM_ARMED_HOME, STATE_ALARM_ARMED_NIGHT as STATE_ALARM_ARMED_NIGHT, STATE_ALARM_DISARMED as STATE_ALARM_DISARMED
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity
from typing import Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class CanaryAlarm(CoordinatorEntity, AlarmControlPanelEntity):
    coordinator: CanaryDataUpdateCoordinator
    _attr_supported_features: Any
    _location_id: Any
    _attr_name: Any
    _attr_unique_id: Any
    def __init__(self, coordinator: CanaryDataUpdateCoordinator, location: Location) -> None: ...
    @property
    def location(self) -> Location: ...
    @property
    def state(self) -> Union[str, None]: ...
    @property
    def extra_state_attributes(self) -> dict[str, Any]: ...
    def alarm_disarm(self, code: Union[str, None] = ...) -> None: ...
    def alarm_arm_home(self, code: Union[str, None] = ...) -> None: ...
    def alarm_arm_away(self, code: Union[str, None] = ...) -> None: ...
    def alarm_arm_night(self, code: Union[str, None] = ...) -> None: ...



from . import AirlyDataUpdateCoordinator as AirlyDataUpdateCoordinator
from .const import ATTRIBUTION as ATTRIBUTION, ATTR_ADVICE as ATTR_ADVICE, ATTR_API_ADVICE as ATTR_API_ADVICE, ATTR_API_CAQI as ATTR_API_CAQI, ATTR_API_CAQI_DESCRIPTION as ATTR_API_CAQI_DESCRIPTION, ATTR_API_CAQI_LEVEL as ATTR_API_CAQI_LEVEL, ATTR_API_PM10 as ATTR_API_PM10, ATTR_API_PM25 as ATTR_API_PM25, ATTR_DESCRIPTION as ATTR_DESCRIPTION, ATTR_LEVEL as ATTR_LEVEL, ATTR_LIMIT as ATTR_LIMIT, ATTR_PERCENT as ATTR_PERCENT, DEFAULT_NAME as DEFAULT_NAME, DOMAIN as DOMAIN, MANUFACTURER as MANUFACTURER, SENSOR_TYPES as SENSOR_TYPES, SUFFIX_LIMIT as SUFFIX_LIMIT, SUFFIX_PERCENT as SUFFIX_PERCENT
from .model import AirlySensorEntityDescription as AirlySensorEntityDescription
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_ATTRIBUTION as ATTR_ATTRIBUTION, CONF_NAME as CONF_NAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import StateType as StateType
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity
from typing import Any

PARALLEL_UPDATES: int

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class AirlySensor(CoordinatorEntity, SensorEntity):
    coordinator: AirlyDataUpdateCoordinator
    entity_description: AirlySensorEntityDescription
    _attr_device_info: Any
    _attr_name: Any
    _attr_unique_id: Any
    _attrs: Any
    def __init__(self, coordinator: AirlyDataUpdateCoordinator, name: str, description: AirlySensorEntityDescription) -> None: ...
    @property
    def state(self) -> StateType: ...
    @property
    def extra_state_attributes(self) -> dict[str, Any]: ...



from typing import Any, Final

COAP: Final[str]
DATA_CONFIG_ENTRY: Final[str]
DEVICE: Final[str]
DOMAIN: Final[str]
REST: Final[str]
CONF_COAP_PORT: Final[str]
DEFAULT_COAP_PORT: Final[int]
POLLING_TIMEOUT_SEC: Final[int]
REST_SENSORS_UPDATE_INTERVAL: Final[int]
AIOSHELLY_DEVICE_TIMEOUT_SEC: Final[int]
SLEEP_PERIOD_MULTIPLIER: Final[float]
UPDATE_PERIOD_MULTIPLIER: Final[float]
SHAIR_MAX_WORK_HOURS: Final[int]
INPUTS_EVENTS_DICT: Final[Any]
BATTERY_DEVICES_WITH_PERMANENT_CONNECTION: Final[Any]
EVENT_SHELLY_CLICK: Final[str]
ATTR_CLICK_TYPE: Final[str]
ATTR_CHANNEL: Final[str]
ATTR_DEVICE: Final[str]
CONF_SUBTYPE: Final[str]
BASIC_INPUTS_EVENTS_TYPES: Final[Any]
SHBTN_INPUTS_EVENTS_TYPES: Final[Any]
SUPPORTED_INPUTS_EVENTS_TYPES: Final[Any]
SHIX3_1_INPUTS_EVENTS_TYPES = SUPPORTED_INPUTS_EVENTS_TYPES
INPUTS_EVENTS_SUBTYPES: Final[Any]
SHBTN_MODELS: Final[Any]
STANDARD_RGB_EFFECTS: Final[Any]
SHBLB_1_RGB_EFFECTS: Final[Any]
KELVIN_MAX_VALUE: Final[int]
KELVIN_MIN_VALUE_WHITE: Final[int]
KELVIN_MIN_VALUE_COLOR: Final[int]
UPTIME_DEVIATION: Final[int]
LAST_RESET_UPTIME: Final[str]
LAST_RESET_NEVER: Final[str]



from homeassistant.const import CONF_DESCRIPTION as CONF_DESCRIPTION, CONF_NAME as CONF_NAME, CONF_PLATFORM as CONF_PLATFORM
from homeassistant.core import HomeAssistant as HomeAssistant, ServiceCall as ServiceCall
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers import config_per_platform as config_per_platform, discovery as discovery
from homeassistant.helpers.service import async_set_service_schema as async_set_service_schema
from homeassistant.loader import async_get_integration as async_get_integration, bind_hass as bind_hass
from homeassistant.setup import async_prepare_setup_platform as async_prepare_setup_platform, async_start_setup as async_start_setup
from homeassistant.util import slugify as slugify
from homeassistant.util.yaml import load_yaml as load_yaml
from typing import Any

_LOGGER: Any
ATTR_DATA: str
ATTR_MESSAGE: str
ATTR_TARGET: str
ATTR_TITLE: str
ATTR_TITLE_DEFAULT: str
DOMAIN: str
SERVICE_NOTIFY: str
SERVICE_PERSISTENT_NOTIFICATION: str
NOTIFY_SERVICES: str
CONF_FIELDS: str
PLATFORM_SCHEMA: Any
NOTIFY_SERVICE_SCHEMA: Any
PERSISTENT_NOTIFICATION_SERVICE_SCHEMA: Any

async def async_reload(hass: HomeAssistant, integration_name: str) -> None: ...
async def async_reset_platform(hass: HomeAssistant, integration_name: str) -> None: ...
def _async_integration_has_notify_services(hass: HomeAssistant, integration_name: str) -> bool: ...

class BaseNotificationService:
    hass: HomeAssistant
    registered_targets: dict[str, str]
    def send_message(self, message, **kwargs) -> None: ...
    async def async_send_message(self, message: Any, **kwargs: Any) -> None: ...
    async def _async_notify_message_service(self, service: ServiceCall) -> None: ...
    _service_name: Any
    _target_service_name_prefix: Any
    services_dict: Any
    async def async_setup(self, hass: HomeAssistant, service_name: str, target_service_name_prefix: str) -> None: ...
    async def async_register_services(self) -> None: ...
    async def async_unregister_services(self) -> None: ...

async def async_setup(hass, config): ...



from .const import DOMAIN as DOMAIN, KNX_ADDRESS as KNX_ADDRESS
from .knx_entity import KnxEntity as KnxEntity
from .schema import SceneSchema as SceneSchema
from homeassistant.components.scene import Scene as Scene
from homeassistant.const import CONF_NAME as CONF_NAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any
from xknx import XKNX as XKNX
from xknx.devices import Scene as XknxScene

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...

class KNXScene(KnxEntity, Scene):
    _device: XknxScene
    _attr_unique_id: Any
    def __init__(self, xknx: XKNX, config: ConfigType) -> None: ...
    async def async_activate(self, **kwargs: Any) -> None: ...



from homeassistant.core import HomeAssistant as HomeAssistant, split_entity_id as split_entity_id
from typing import Any

_LOGGER: Any

def is_on(hass: HomeAssistant, entity_id: Union[str, None] = ...) -> bool: ...



from .const import CONF_TRACK_WIRED_CLIENTS as CONF_TRACK_WIRED_CLIENTS, CONF_UNAUTHENTICATED_MODE as CONF_UNAUTHENTICATED_MODE, CONNECTION_TIMEOUT as CONNECTION_TIMEOUT, DEFAULT_DEVICE_NAME as DEFAULT_DEVICE_NAME, DEFAULT_NOTIFY_SERVICE_NAME as DEFAULT_NOTIFY_SERVICE_NAME, DEFAULT_TRACK_WIRED_CLIENTS as DEFAULT_TRACK_WIRED_CLIENTS, DEFAULT_UNAUTHENTICATED_MODE as DEFAULT_UNAUTHENTICATED_MODE, DOMAIN as DOMAIN
from .utils import get_device_macs as get_device_macs
from homeassistant import config_entries as config_entries
from homeassistant.components import ssdp as ssdp
from homeassistant.const import CONF_MAC as CONF_MAC, CONF_NAME as CONF_NAME, CONF_PASSWORD as CONF_PASSWORD, CONF_RECIPIENT as CONF_RECIPIENT, CONF_URL as CONF_URL, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.typing import DiscoveryInfoType as DiscoveryInfoType
from huawei_lte_api.Connection import GetResponseType as GetResponseType
from typing import Any

_LOGGER: Any

class ConfigFlowHandler(config_entries.ConfigFlow):
    VERSION: int
    @staticmethod
    def async_get_options_flow(config_entry: config_entries.ConfigEntry) -> OptionsFlowHandler: ...
    async def _async_show_user_form(self, user_input: Union[dict[str, Any], None] = ..., errors: Union[dict[str, str], None] = ...) -> FlowResult: ...
    async def async_step_import(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_ssdp(self, discovery_info: DiscoveryInfoType) -> FlowResult: ...

class OptionsFlowHandler(config_entries.OptionsFlow):
    config_entry: Any
    def __init__(self, config_entry: config_entries.ConfigEntry) -> None: ...
    async def async_step_init(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...



from aiohttp import web
from homeassistant.components.http import HomeAssistantView as HomeAssistantView
from homeassistant.const import HTTP_INTERNAL_SERVER_ERROR as HTTP_INTERNAL_SERVER_ERROR
from homeassistant.core import callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers import config_per_platform as config_per_platform, discovery as discovery
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.setup import async_prepare_setup_platform as async_prepare_setup_platform
from typing import Any

_LOGGER: Any
DOMAIN: str
EVENT: str
CONTENT_TYPE_MPEG: str
CONTENT_TYPE_NONE: str
SCAN_INTERVAL: Any

async def async_setup(hass, config): ...

class MailboxEntity(Entity):
    mailbox: Any
    message_count: int
    def __init__(self, mailbox) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    @property
    def state(self): ...
    @property
    def name(self): ...
    async def async_update(self) -> None: ...

class Mailbox:
    hass: Any
    name: Any
    def __init__(self, hass, name) -> None: ...
    def async_update(self) -> None: ...
    @property
    def media_type(self) -> None: ...
    @property
    def can_delete(self): ...
    @property
    def has_media(self): ...
    async def async_get_media(self, msgid) -> None: ...
    async def async_get_messages(self) -> None: ...
    async def async_delete(self, msgid) -> None: ...

class StreamError(Exception): ...

class MailboxView(HomeAssistantView):
    mailboxes: Any
    def __init__(self, mailboxes) -> None: ...
    def get_mailbox(self, platform): ...

class MailboxPlatformsView(MailboxView):
    url: str
    name: str
    async def get(self, request: web.Request) -> web.Response: ...

class MailboxMessageView(MailboxView):
    url: str
    name: str
    async def get(self, request, platform): ...

class MailboxDeleteView(MailboxView):
    url: str
    name: str
    async def delete(self, request, platform, msgid) -> None: ...

class MailboxMediaView(MailboxView):
    url: str
    name: str
    async def get(self, request, platform, msgid): ...



from .entry_data import RuntimeEntryData as RuntimeEntryData
from aioesphomeapi import APIClient, APIIntEnum, APIVersion as APIVersion, DeviceInfo as EsphomeDeviceInfo, EntityInfo, EntityState, HomeassistantServiceCall as HomeassistantServiceCall, UserService as UserService
from collections.abc import Awaitable
from homeassistant import const as const
from homeassistant.components import zeroconf as zeroconf
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_HOST as CONF_HOST, CONF_MODE as CONF_MODE, CONF_PASSWORD as CONF_PASSWORD, CONF_PORT as CONF_PORT, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, ServiceCall as ServiceCall, State as State, callback as callback
from homeassistant.exceptions import TemplateError as TemplateError
from homeassistant.helpers import template as template
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo, Entity as Entity
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.event import async_track_state_change_event as async_track_state_change_event
from homeassistant.helpers.json import JSONEncoder as JSONEncoder
from homeassistant.helpers.service import async_set_service_schema as async_set_service_schema
from homeassistant.helpers.storage import Store as Store
from homeassistant.helpers.template import Template as Template
from typing import Any, Callable, TypeVar, overload
from zeroconf import DNSRecord as DNSRecord, RecordUpdateListener, Zeroconf as Zeroconf

DOMAIN: str
_LOGGER: Any
_T = TypeVar('_T')
STORAGE_VERSION: int

class DomainData:
    _entry_datas: dict[str, RuntimeEntryData]
    _stores: dict[str, Store]
    def get_entry_data(self, entry: ConfigEntry) -> RuntimeEntryData: ...
    def set_entry_data(self, entry: ConfigEntry, entry_data: RuntimeEntryData) -> None: ...
    def pop_entry_data(self, entry: ConfigEntry) -> RuntimeEntryData: ...
    def is_entry_loaded(self, entry: ConfigEntry) -> bool: ...
    def get_or_create_store(self, hass: HomeAssistant, entry: ConfigEntry) -> Store: ...
    @classmethod
    def get(cls, hass: HomeAssistant) -> _T: ...

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class ReconnectLogic(RecordUpdateListener):
    _hass: Any
    _cli: Any
    _entry: Any
    _host: Any
    _on_login: Any
    _zc: Any
    _connected: bool
    _connected_lock: Any
    _zc_lock: Any
    _zc_listening: bool
    _reconnect_event: Any
    _loop_task: Any
    _tries: int
    _tries_lock: Any
    _wait_task: Any
    _wait_task_lock: Any
    def __init__(self, hass: HomeAssistant, cli: APIClient, entry: ConfigEntry, host: str, on_login: Callable[[], Awaitable[None]], zc: Zeroconf) -> None: ...
    @property
    def _entry_data(self) -> Union[RuntimeEntryData, None]: ...
    async def _on_disconnect(self) -> None: ...
    async def _wait_and_start_reconnect(self) -> None: ...
    async def _try_connect(self) -> None: ...
    async def _reconnect_once(self) -> None: ...
    async def _reconnect_loop(self) -> None: ...
    async def start(self) -> None: ...
    async def stop(self) -> None: ...
    async def _start_zc_listen(self) -> None: ...
    async def _stop_zc_listen(self) -> None: ...
    def stop_callback(self) -> None: ...
    def _set_reconnect(self) -> None: ...
    def update_record(self, zc: Zeroconf, now: float, record: DNSRecord) -> None: ...

async def _async_setup_device_registry(hass: HomeAssistant, entry: ConfigEntry, device_info: EsphomeDeviceInfo) -> str: ...

ARG_TYPE_METADATA: Any

async def _register_service(hass: HomeAssistant, entry_data: RuntimeEntryData, service: UserService) -> None: ...
async def _setup_services(hass: HomeAssistant, entry_data: RuntimeEntryData, services: list[UserService]) -> None: ...
async def _cleanup_instance(hass: HomeAssistant, entry: ConfigEntry) -> RuntimeEntryData: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_remove_entry(hass: HomeAssistant, entry: ConfigEntry) -> None: ...
_InfoT = TypeVar('_InfoT', bound=EntityInfo)
_EntityT = TypeVar('_EntityT', bound='EsphomeEntity[Any,Any]')
_StateT = TypeVar('_StateT', bound=EntityState)

async def platform_async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback, component_key: str, info_type: type[_InfoT], entity_type: type[_EntityT], state_type: type[_StateT]) -> None: ...
_PropT = TypeVar('_PropT', bound=Callable[..., Any])

def esphome_state_property(func: _PropT) -> _PropT: ...
_EnumT = TypeVar('_EnumT', bound=APIIntEnum)
_ValT = TypeVar('_ValT')

class EsphomeEnumMapper:
    _mapping: Any
    _inverse: Any
    def __init__(self, mapping: dict[_EnumT, _ValT]) -> None: ...
    @overload
    def from_esphome(self, value: _EnumT) -> _ValT: ...
    @overload
    def from_esphome(self, value: Union[_EnumT, None]) -> Union[_ValT, None]: ...
    def from_hass(self, value: _ValT) -> _EnumT: ...

class EsphomeEntity(Entity):
    _entry_data: Any
    _component_key: Any
    _key: Any
    def __init__(self, entry_data: RuntimeEntryData, component_key: str, key: int) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    def _on_state_update(self) -> None: ...
    def _on_device_update(self) -> None: ...
    @property
    def _entry_id(self) -> str: ...
    @property
    def _api_version(self) -> APIVersion: ...
    @property
    def _static_info(self) -> _InfoT: ...
    @property
    def _device_info(self) -> EsphomeDeviceInfo: ...
    @property
    def _client(self) -> APIClient: ...
    @property
    def _state(self) -> _StateT: ...
    @property
    def _has_state(self) -> bool: ...
    @property
    def available(self) -> bool: ...
    @property
    def unique_id(self) -> Union[str, None]: ...
    @property
    def device_info(self) -> DeviceInfo: ...
    @property
    def name(self) -> str: ...
    @property
    def should_poll(self) -> bool: ...



from . import SynoApi as SynoApi, SynologyDSMBaseEntity as SynologyDSMBaseEntity
from .const import COORDINATOR_SWITCHES as COORDINATOR_SWITCHES, DOMAIN as DOMAIN, EntityInfo as EntityInfo, SURVEILLANCE_SWITCH as SURVEILLANCE_SWITCH, SYNO_API as SYNO_API
from homeassistant.components.switch import ToggleEntity as ToggleEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator
from typing import Any

_LOGGER: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class SynoDSMSurveillanceHomeModeToggle(SynologyDSMBaseEntity, ToggleEntity):
    coordinator: DataUpdateCoordinator[dict[str, dict[str, bool]]]
    _version: Any
    def __init__(self, api: SynoApi, entity_type: str, entity_info: EntityInfo, version: str, coordinator: DataUpdateCoordinator[dict[str, dict[str, bool]]]) -> None: ...
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    @property
    def available(self) -> bool: ...
    @property
    def device_info(self) -> DeviceInfo: ...



import logging
from collections.abc import Awaitable
from datetime import timedelta
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator, UpdateFailed as UpdateFailed
from typing import Any, Callable, TypeVar

T = TypeVar('T')

class RenaultDataUpdateCoordinator(DataUpdateCoordinator[T]):
    access_denied: bool
    not_supported: bool
    def __init__(self, hass: HomeAssistant, logger: logging.Logger, name: str, update_interval: timedelta, update_method: Callable[[], Awaitable[T]]) -> None: ...
    update_interval: Any
    async def _async_update_data(self) -> T: ...
    async def async_config_entry_first_refresh(self) -> None: ...



from . import OpenUV as OpenUV, OpenUvEntity as OpenUvEntity
from .const import DATA_CLIENT as DATA_CLIENT, DATA_UV as DATA_UV, DOMAIN as DOMAIN, TYPE_CURRENT_OZONE_LEVEL as TYPE_CURRENT_OZONE_LEVEL, TYPE_CURRENT_UV_INDEX as TYPE_CURRENT_UV_INDEX, TYPE_CURRENT_UV_LEVEL as TYPE_CURRENT_UV_LEVEL, TYPE_MAX_UV_INDEX as TYPE_MAX_UV_INDEX, TYPE_SAFE_EXPOSURE_TIME_1 as TYPE_SAFE_EXPOSURE_TIME_1, TYPE_SAFE_EXPOSURE_TIME_2 as TYPE_SAFE_EXPOSURE_TIME_2, TYPE_SAFE_EXPOSURE_TIME_3 as TYPE_SAFE_EXPOSURE_TIME_3, TYPE_SAFE_EXPOSURE_TIME_4 as TYPE_SAFE_EXPOSURE_TIME_4, TYPE_SAFE_EXPOSURE_TIME_5 as TYPE_SAFE_EXPOSURE_TIME_5, TYPE_SAFE_EXPOSURE_TIME_6 as TYPE_SAFE_EXPOSURE_TIME_6
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import TIME_MINUTES as TIME_MINUTES, UV_INDEX as UV_INDEX
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.util.dt import as_local as as_local, parse_datetime as parse_datetime
from typing import Any

ATTR_MAX_UV_TIME: str
EXPOSURE_TYPE_MAP: Any
UV_LEVEL_EXTREME: str
UV_LEVEL_VHIGH: str
UV_LEVEL_HIGH: str
UV_LEVEL_MODERATE: str
UV_LEVEL_LOW: str
SENSORS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class OpenUvSensor(OpenUvEntity, SensorEntity):
    _attr_icon: Any
    _attr_name: Any
    _attr_unit_of_measurement: Any
    def __init__(self, openuv: OpenUV, sensor_type: str, name: str, icon: str, unit: Union[str, None]) -> None: ...
    _attr_available: bool
    _attr_state: Any
    def update_from_latest_data(self) -> None: ...



from .models import HaZeroconf as HaZeroconf
from homeassistant.helpers.frame import MissingIntegrationFrame as MissingIntegrationFrame, get_integration_frame as get_integration_frame, report_integration as report_integration
from typing import Any

_LOGGER: Any

def install_multiple_zeroconf_catcher(hass_zc: HaZeroconf) -> None: ...
def _report(what: str) -> None: ...



from .const import ACCOUNT_HASH as ACCOUNT_HASH, DOMAIN as DOMAIN
from homeassistant import config_entries as config_entries
from homeassistant.const import CONF_EMAIL as CONF_EMAIL, CONF_PASSWORD as CONF_PASSWORD
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from typing import Any

_LOGGER: Any
DATA_SCHEMA: Any

class ConfigFlow(config_entries.ConfigFlow):
    VERSION: int
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...



from .const import ATTR_ENDPOINTS as ATTR_ENDPOINTS, ATTR_STREAMS as ATTR_STREAMS, DOMAIN as DOMAIN, HLS_PROVIDER as HLS_PROVIDER, MAX_SEGMENTS as MAX_SEGMENTS, OUTPUT_IDLE_TIMEOUT as OUTPUT_IDLE_TIMEOUT, RECORDER_PROVIDER as RECORDER_PROVIDER, STREAM_RESTART_INCREMENT as STREAM_RESTART_INCREMENT, STREAM_RESTART_RESET_TIME as STREAM_RESTART_RESET_TIME
from .core import IdleTimer as IdleTimer, PROVIDERS as PROVIDERS, StreamOutput as StreamOutput
from .hls import async_setup_hls as async_setup_hls
from collections.abc import Mapping
from homeassistant.const import EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any

_LOGGER: Any
STREAM_SOURCE_RE: Any

def redact_credentials(data: str) -> str: ...
def create_stream(hass: HomeAssistant, stream_source: str, options: dict[str, str]) -> Stream: ...
async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...

class Stream:
    hass: Any
    source: Any
    options: Any
    keepalive: bool
    access_token: Any
    _thread: Any
    _thread_quit: Any
    _outputs: Any
    _fast_restart_once: bool
    def __init__(self, hass: HomeAssistant, source: str, options: dict[str, str]) -> None: ...
    def endpoint_url(self, fmt: str) -> str: ...
    def outputs(self) -> Mapping[str, StreamOutput]: ...
    def add_provider(self, fmt: str, timeout: int = ...) -> StreamOutput: ...
    def remove_provider(self, provider: StreamOutput) -> None: ...
    def check_idle(self) -> None: ...
    def start(self) -> None: ...
    def update_source(self, new_source: str) -> None: ...
    def _run_worker(self) -> None: ...
    def _worker_finished(self) -> None: ...
    def stop(self) -> None: ...
    def _stop(self) -> None: ...
    async def async_record(self, video_path: str, duration: int = ..., lookback: int = ...) -> None: ...



from . import PairedSensorEntity as PairedSensorEntity, ValveControllerEntity as ValveControllerEntity
from .const import API_SYSTEM_ONBOARD_SENSOR_STATUS as API_SYSTEM_ONBOARD_SENSOR_STATUS, API_WIFI_STATUS as API_WIFI_STATUS, CONF_UID as CONF_UID, DATA_COORDINATOR as DATA_COORDINATOR, DATA_COORDINATOR_PAIRED_SENSOR as DATA_COORDINATOR_PAIRED_SENSOR, DATA_UNSUB_DISPATCHER_CONNECT as DATA_UNSUB_DISPATCHER_CONNECT, DOMAIN as DOMAIN, SIGNAL_PAIRED_SENSOR_COORDINATOR_ADDED as SIGNAL_PAIRED_SENSOR_COORDINATOR_ADDED
from homeassistant.components.binary_sensor import BinarySensorEntity as BinarySensorEntity, DEVICE_CLASS_CONNECTIVITY as DEVICE_CLASS_CONNECTIVITY, DEVICE_CLASS_MOISTURE as DEVICE_CLASS_MOISTURE, DEVICE_CLASS_MOVING as DEVICE_CLASS_MOVING
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator
from typing import Any

ATTR_CONNECTED_CLIENTS: str
SENSOR_KIND_AP_INFO: str
SENSOR_KIND_LEAK_DETECTED: str
SENSOR_KIND_MOVED: str
SENSOR_ATTRS_MAP: Any
PAIRED_SENSOR_SENSORS: Any
VALVE_CONTROLLER_SENSORS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class PairedSensorBinarySensor(PairedSensorEntity, BinarySensorEntity):
    _attr_is_on: bool
    def __init__(self, entry: ConfigEntry, coordinator: DataUpdateCoordinator, kind: str, name: str, device_class: Union[str, None], icon: Union[str, None]) -> None: ...
    def _async_update_from_latest_data(self) -> None: ...

class ValveControllerBinarySensor(ValveControllerEntity, BinarySensorEntity):
    _attr_is_on: bool
    def __init__(self, entry: ConfigEntry, coordinators: dict[str, DataUpdateCoordinator], kind: str, name: str, device_class: Union[str, None], icon: Union[str, None]) -> None: ...
    async def _async_continue_entity_setup(self) -> None: ...
    _attr_available: Any
    def _async_update_from_latest_data(self) -> None: ...



from .const import ATTR_MANUFACTURER as ATTR_MANUFACTURER, DEFAULT_NAME as DEFAULT_NAME, DOMAIN as DOMAIN
from homeassistant.components.media_player import MediaPlayerEntity as MediaPlayerEntity
from homeassistant.components.media_player.const import SUPPORT_NEXT_TRACK as SUPPORT_NEXT_TRACK, SUPPORT_PAUSE as SUPPORT_PAUSE, SUPPORT_PLAY as SUPPORT_PLAY, SUPPORT_PREVIOUS_TRACK as SUPPORT_PREVIOUS_TRACK, SUPPORT_TURN_OFF as SUPPORT_TURN_OFF, SUPPORT_TURN_ON as SUPPORT_TURN_ON
from homeassistant.config_entries import ConfigEntry as ConfigEntry, SOURCE_IMPORT as SOURCE_IMPORT
from homeassistant.const import CONF_HOST as CONF_HOST, CONF_NAME as CONF_NAME, STATE_OFF as STATE_OFF, STATE_ON as STATE_ON, STATE_PAUSED as STATE_PAUSED, STATE_PLAYING as STATE_PLAYING
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from pdunehd import DuneHDPlayer as DuneHDPlayer
from typing import Any, Final

CONF_SOURCES: Final[str]
PLATFORM_SCHEMA: Final[Any]
DUNEHD_PLAYER_SUPPORT: Final[int]

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class DuneHDPlayerEntity(MediaPlayerEntity):
    _player: Any
    _name: Any
    _media_title: Any
    _state: Any
    _unique_id: Any
    def __init__(self, player: DuneHDPlayer, name: str, unique_id: str) -> None: ...
    def update(self) -> bool: ...
    @property
    def state(self) -> Union[str, None]: ...
    @property
    def name(self) -> str: ...
    @property
    def available(self) -> bool: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def device_info(self) -> DeviceInfo: ...
    @property
    def volume_level(self) -> float: ...
    @property
    def is_volume_muted(self) -> bool: ...
    @property
    def supported_features(self) -> int: ...
    def volume_up(self) -> None: ...
    def volume_down(self) -> None: ...
    def mute_volume(self, mute: bool) -> None: ...
    def turn_off(self) -> None: ...
    def turn_on(self) -> None: ...
    def media_play(self) -> None: ...
    def media_pause(self) -> None: ...
    @property
    def media_title(self) -> Union[str, None]: ...
    def __update_title(self) -> None: ...
    def media_previous_track(self) -> None: ...
    def media_next_track(self) -> None: ...



from .const import KNX_ADDRESS as KNX_ADDRESS
from .schema import ExposeSchema as ExposeSchema
from homeassistant.const import CONF_ENTITY_ID as CONF_ENTITY_ID, STATE_OFF as STATE_OFF, STATE_ON as STATE_ON, STATE_UNAVAILABLE as STATE_UNAVAILABLE, STATE_UNKNOWN as STATE_UNKNOWN
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, State as State, callback as callback
from homeassistant.helpers.event import async_track_state_change_event as async_track_state_change_event
from homeassistant.helpers.typing import ConfigType as ConfigType, StateType as StateType
from typing import Any
from xknx import XKNX as XKNX
from xknx.devices import DateTime, ExposeSensor

def create_knx_exposure(hass: HomeAssistant, xknx: XKNX, config: ConfigType) -> Union[KNXExposeSensor, KNXExposeTime]: ...

class KNXExposeSensor:
    hass: Any
    xknx: Any
    type: Any
    entity_id: Any
    expose_attribute: Any
    expose_default: Any
    address: Any
    _remove_listener: Any
    device: Any
    def __init__(self, hass: HomeAssistant, xknx: XKNX, expose_type: Union[int, str], entity_id: str, attribute: Union[str, None], default: StateType, address: str) -> None: ...
    def async_register(self) -> ExposeSensor: ...
    def _init_expose_state(self) -> None: ...
    def shutdown(self) -> None: ...
    def _get_expose_value(self, state: Union[State, None]) -> StateType: ...
    async def _async_entity_changed(self, event: Event) -> None: ...
    async def _async_set_knx_value(self, value: StateType) -> None: ...

class KNXExposeTime:
    xknx: Any
    expose_type: Any
    address: Any
    device: Any
    def __init__(self, xknx: XKNX, expose_type: str, address: str) -> None: ...
    def async_register(self) -> DateTime: ...
    def shutdown(self) -> None: ...



import av
from . import redact_credentials as redact_credentials
from .const import AUDIO_CODECS as AUDIO_CODECS, MAX_MISSING_DTS as MAX_MISSING_DTS, MAX_TIMESTAMP_GAP as MAX_TIMESTAMP_GAP, MIN_SEGMENT_DURATION as MIN_SEGMENT_DURATION, PACKETS_TO_WAIT_FOR_AUDIO as PACKETS_TO_WAIT_FOR_AUDIO, SEGMENT_CONTAINER_FORMAT as SEGMENT_CONTAINER_FORMAT, SOURCE_TIMEOUT as SOURCE_TIMEOUT, TARGET_PART_DURATION as TARGET_PART_DURATION
from .core import Part as Part, Segment as Segment, StreamOutput as StreamOutput
from collections.abc import Generator, Iterator, Mapping
from io import BytesIO
from threading import Event
from typing import Any, Callable

_LOGGER: Any

class SegmentBuffer:
    _stream_id: int
    _outputs_callback: Any
    _sequence: int
    _segment_start_dts: Any
    _memory_file: Any
    _av_output: Any
    _input_video_stream: Any
    _input_audio_stream: Any
    _output_video_stream: Any
    _output_audio_stream: Any
    _segment: Any
    _memory_file_pos: Any
    _part_start_dts: Any
    _part_has_keyframe: bool
    def __init__(self, outputs_callback: Callable[[], Mapping[str, StreamOutput]]) -> None: ...
    @staticmethod
    def make_new_av(memory_file: BytesIO, sequence: int, input_vstream: av.video.VideoStream) -> av.container.OutputContainer: ...
    def set_streams(self, video_stream: av.video.VideoStream, audio_stream: Any) -> None: ...
    def reset(self, video_dts: int) -> None: ...
    def mux_packet(self, packet: av.Packet) -> None: ...
    def check_flush_part(self, packet: av.Packet) -> None: ...
    def flush(self, packet: av.Packet, last_part: bool) -> None: ...
    def discontinuity(self) -> None: ...
    def close(self) -> None: ...

class PeekIterator(Iterator):
    _iterator: Any
    _buffer: Any
    _next: Any
    def __init__(self, iterator: Iterator[av.Packet]) -> None: ...
    def __iter__(self) -> Iterator: ...
    def __next__(self) -> av.Packet: ...
    def replace_underlying_iterator(self, new_iterator: Iterator) -> None: ...
    def _pop_buffer(self) -> av.Packet: ...
    def peek(self) -> Generator[av.Packet, None, None]: ...

class TimestampValidator:
    _last_dts: Any
    _missing_dts: int
    def __init__(self): ...
    def is_valid(self, packet: av.Packet) -> bool: ...

def is_keyframe(packet: av.Packet) -> Any: ...
def unsupported_audio(packets: Iterator[av.Packet], audio_stream: Any) -> bool: ...
def stream_worker(source: str, options: dict[str, str], segment_buffer: SegmentBuffer, quit_event: Event) -> None: ...



from .const import CATEGORY_CDC_REPORT as CATEGORY_CDC_REPORT, CATEGORY_USER_REPORT as CATEGORY_USER_REPORT, DATA_COORDINATOR as DATA_COORDINATOR, DOMAIN as DOMAIN
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_ATTRIBUTION as ATTR_ATTRIBUTION, ATTR_STATE as ATTR_STATE, CONF_LATITUDE as CONF_LATITUDE, CONF_LONGITUDE as CONF_LONGITUDE
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity, DataUpdateCoordinator as DataUpdateCoordinator
from typing import Any

ATTR_CITY: str
ATTR_REPORTED_DATE: str
ATTR_REPORTED_LATITUDE: str
ATTR_REPORTED_LONGITUDE: str
ATTR_STATE_REPORTS_LAST_WEEK: str
ATTR_STATE_REPORTS_THIS_WEEK: str
ATTR_ZIP_CODE: str
DEFAULT_ATTRIBUTION: str
SENSOR_TYPE_CDC_LEVEL: str
SENSOR_TYPE_CDC_LEVEL2: str
SENSOR_TYPE_USER_CHICK: str
SENSOR_TYPE_USER_DENGUE: str
SENSOR_TYPE_USER_FLU: str
SENSOR_TYPE_USER_LEPTO: str
SENSOR_TYPE_USER_NO_SYMPTOMS: str
SENSOR_TYPE_USER_SYMPTOMS: str
SENSOR_TYPE_USER_TOTAL: str
CDC_SENSORS: Any
USER_SENSORS: Any
EXTENDED_SENSOR_TYPE_MAPPING: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class FluNearYouSensor(CoordinatorEntity, SensorEntity):
    _attr_extra_state_attributes: Any
    _attr_icon: Any
    _attr_name: Any
    _attr_unique_id: Any
    _attr_unit_of_measurement: Any
    _entry: Any
    _sensor_type: Any
    def __init__(self, coordinator: DataUpdateCoordinator, entry: ConfigEntry, sensor_type: str, name: str, icon: str, unit: Union[str, None]) -> None: ...
    def _handle_coordinator_update(self) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    def update_from_latest_data(self) -> None: ...

class CdcSensor(FluNearYouSensor):
    _attr_state: Any
    def update_from_latest_data(self) -> None: ...

class UserSensor(FluNearYouSensor):
    _attr_state: Any
    def update_from_latest_data(self) -> None: ...



from typing import TypeVar

T = TypeVar('T')

def ordered_list_item_to_percentage(ordered_list: list[T], item: T) -> int: ...
def percentage_to_ordered_list_item(ordered_list: list[T], percentage: int) -> T: ...
def ranged_value_to_percentage(low_high_range: tuple[float, float], value: float) -> int: ...
def percentage_to_ranged_value(low_high_range: tuple[float, float], percentage: int) -> float: ...
def states_in_range(low_high_range: tuple[float, float]) -> float: ...
def int_states_in_range(low_high_range: tuple[float, float]) -> int: ...



from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, ATTR_TEMPERATURE as ATTR_TEMPERATURE, PRECISION_TENTHS as PRECISION_TENTHS, PRECISION_WHOLE as PRECISION_WHOLE, SERVICE_TURN_OFF as SERVICE_TURN_OFF, SERVICE_TURN_ON as SERVICE_TURN_ON, STATE_OFF as STATE_OFF, STATE_ON as STATE_ON, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE
from homeassistant.helpers.entity import Entity as Entity, EntityDescription as EntityDescription
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from typing import Any

DEFAULT_MIN_TEMP: int
DEFAULT_MAX_TEMP: int
DOMAIN: str
ENTITY_ID_FORMAT: Any
SCAN_INTERVAL: Any
SERVICE_SET_AWAY_MODE: str
SERVICE_SET_TEMPERATURE: str
SERVICE_SET_OPERATION_MODE: str
STATE_ECO: str
STATE_ELECTRIC: str
STATE_PERFORMANCE: str
STATE_HIGH_DEMAND: str
STATE_HEAT_PUMP: str
STATE_GAS: str
SUPPORT_TARGET_TEMPERATURE: int
SUPPORT_OPERATION_MODE: int
SUPPORT_AWAY_MODE: int
ATTR_MAX_TEMP: str
ATTR_MIN_TEMP: str
ATTR_AWAY_MODE: str
ATTR_OPERATION_MODE: str
ATTR_OPERATION_LIST: str
ATTR_TARGET_TEMP_HIGH: str
ATTR_TARGET_TEMP_LOW: str
ATTR_CURRENT_TEMPERATURE: str
CONVERTIBLE_ATTRIBUTE: Any
_LOGGER: Any
ON_OFF_SERVICE_SCHEMA: Any
SET_AWAY_MODE_SCHEMA: Any
SET_TEMPERATURE_SCHEMA: Any
SET_OPERATION_MODE_SCHEMA: Any

async def async_setup(hass, config): ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class WaterHeaterEntityEntityDescription(EntityDescription): ...

class WaterHeaterEntity(Entity):
    entity_description: WaterHeaterEntityEntityDescription
    _attr_current_operation: Union[str, None]
    _attr_current_temperature: Union[float, None]
    _attr_is_away_mode_on: Union[bool, None]
    _attr_max_temp: float
    _attr_min_temp: float
    _attr_operation_list: Union[list[str], None]
    _attr_precision: float
    _attr_state: None
    _attr_supported_features: int
    _attr_target_temperature_high: Union[float, None]
    _attr_target_temperature_low: Union[float, None]
    _attr_target_temperature: Union[float, None]
    _attr_temperature_unit: str
    @property
    def state(self) -> Union[str, None]: ...
    @property
    def precision(self) -> float: ...
    @property
    def capability_attributes(self): ...
    @property
    def state_attributes(self): ...
    @property
    def temperature_unit(self) -> str: ...
    @property
    def current_operation(self) -> Union[str, None]: ...
    @property
    def operation_list(self) -> Union[list[str], None]: ...
    @property
    def current_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature_high(self) -> Union[float, None]: ...
    @property
    def target_temperature_low(self) -> Union[float, None]: ...
    @property
    def is_away_mode_on(self) -> Union[bool, None]: ...
    def set_temperature(self, **kwargs) -> None: ...
    async def async_set_temperature(self, **kwargs) -> None: ...
    def set_operation_mode(self, operation_mode) -> None: ...
    async def async_set_operation_mode(self, operation_mode) -> None: ...
    def turn_away_mode_on(self) -> None: ...
    async def async_turn_away_mode_on(self) -> None: ...
    def turn_away_mode_off(self) -> None: ...
    async def async_turn_away_mode_off(self) -> None: ...
    @property
    def min_temp(self): ...
    @property
    def max_temp(self): ...

async def async_service_away_mode(entity, service) -> None: ...
async def async_service_temperature_set(entity, service) -> None: ...

class WaterHeaterDevice(WaterHeaterEntity):
    def __init_subclass__(cls, **kwargs) -> None: ...



from .const import DEFAULT_DEVICE_NAME as DEFAULT_DEVICE_NAME, DEFAULT_HOST as DEFAULT_HOST, DEFAULT_PORT as DEFAULT_PORT, DEFAULT_USERNAME as DEFAULT_USERNAME, DOMAIN as DOMAIN, SERVICE_REBOOT as SERVICE_REBOOT, SERVICE_RECONNECT as SERVICE_RECONNECT, TRACKER_SCAN_INTERVAL as TRACKER_SCAN_INTERVAL
from datetime import datetime
from homeassistant.components.device_tracker.const import CONF_CONSIDER_HOME as CONF_CONSIDER_HOME, DEFAULT_CONSIDER_HOME as DEFAULT_CONSIDER_HOME
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers.device_registry import CONNECTION_NETWORK_MAC as CONNECTION_NETWORK_MAC
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect, dispatcher_send as dispatcher_send
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo, Entity as Entity
from homeassistant.helpers.event import async_track_time_interval as async_track_time_interval
from types import MappingProxyType
from typing import Any, Callable, TypedDict

_LOGGER: Any

class ClassSetupMissing(Exception):
    def __init__(self) -> None: ...

class Device:
    mac: str
    ip_address: str
    name: str

class HostInfo(TypedDict):
    mac: str
    name: str
    ip: str
    status: bool

class FritzBoxTools:
    _cancel_scan: Any
    _devices: Any
    _options: Any
    _unique_id: Any
    connection: Any
    fritz_hosts: Any
    fritz_status: Any
    hass: Any
    host: Any
    password: Any
    port: Any
    username: Any
    _mac: Any
    _model: Any
    _sw_version: Any
    def __init__(self, hass: HomeAssistant, password: str, username: str = ..., host: str = ..., port: int = ...) -> None: ...
    async def async_setup(self) -> None: ...
    def setup(self) -> None: ...
    async def async_start(self, options: MappingProxyType[str, Any]) -> None: ...
    def async_unload(self) -> None: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def model(self) -> str: ...
    @property
    def sw_version(self) -> str: ...
    @property
    def mac(self) -> str: ...
    @property
    def devices(self) -> dict[str, Any]: ...
    @property
    def signal_device_new(self) -> str: ...
    @property
    def signal_device_update(self) -> str: ...
    def _update_info(self) -> list[HostInfo]: ...
    def scan_devices(self, now: Union[datetime, None] = ...) -> None: ...
    async def service_fritzbox(self, service: str) -> None: ...

class FritzData:
    tracked: dict
    profile_switches: dict

class FritzDeviceBase(Entity):
    _router: Any
    _mac: Any
    _name: Any
    def __init__(self, router: FritzBoxTools, device: FritzDevice) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def ip_address(self) -> Union[str, None]: ...
    @property
    def mac_address(self) -> str: ...
    @property
    def hostname(self) -> Union[str, None]: ...
    @property
    def device_info(self) -> DeviceInfo: ...
    @property
    def should_poll(self) -> bool: ...
    @property
    def entity_registry_enabled_default(self) -> bool: ...
    async def async_process_update(self) -> None: ...
    async def async_on_demand_update(self) -> None: ...
    async def async_added_to_hass(self) -> None: ...

class FritzDevice:
    _mac: Any
    _name: Any
    _ip_address: Any
    _last_activity: Any
    _connected: bool
    def __init__(self, mac: str, name: str) -> None: ...
    def update(self, dev_info: Device, dev_home: bool, consider_home: float) -> None: ...
    @property
    def is_connected(self) -> bool: ...
    @property
    def mac_address(self) -> str: ...
    @property
    def hostname(self) -> str: ...
    @property
    def ip_address(self) -> Union[str, None]: ...
    @property
    def last_activity(self) -> Union[datetime, None]: ...

class SwitchInfo(TypedDict):
    description: str
    friendly_name: str
    icon: str
    type: str
    callback_update: Callable
    callback_switch: Callable

class FritzBoxBaseEntity:
    _fritzbox_tools: Any
    _device_name: Any
    def __init__(self, fritzbox_tools: FritzBoxTools, device_name: str) -> None: ...
    @property
    def mac_address(self) -> str: ...
    @property
    def device_info(self) -> DeviceInfo: ...



from typing import Any, Final

LOGGER: Final[Any]
DOMAIN: Final[str]
PLATFORM_TYPE_LEGACY: Final[str]
PLATFORM_TYPE_ENTITY: Final[str]
SOURCE_TYPE_GPS: Final[str]
SOURCE_TYPE_ROUTER: Final[str]
SOURCE_TYPE_BLUETOOTH: Final[str]
SOURCE_TYPE_BLUETOOTH_LE: Final[str]
CONF_SCAN_INTERVAL: Final[str]
SCAN_INTERVAL: Final[Any]
CONF_TRACK_NEW: Final[str]
DEFAULT_TRACK_NEW: Final[bool]
CONF_CONSIDER_HOME: Final[str]
DEFAULT_CONSIDER_HOME: Final[Any]
CONF_NEW_DEVICE_DEFAULTS: Final[str]
ATTR_ATTRIBUTES: Final[str]
ATTR_BATTERY: Final[str]
ATTR_DEV_ID: Final[str]
ATTR_GPS: Final[str]
ATTR_HOST_NAME: Final[str]
ATTR_LOCATION_NAME: Final[str]
ATTR_MAC: Final[str]
ATTR_SOURCE_TYPE: Final[str]
ATTR_CONSIDER_HOME: Final[str]
ATTR_IP: Final[str]



import asyncio
import logging
from .trace import TraceElement as TraceElement, async_trace_path as async_trace_path, trace_append_element as trace_append_element, trace_id_get as trace_id_get, trace_path as trace_path, trace_path_get as trace_path_get, trace_set_result as trace_set_result, trace_stack_cv as trace_stack_cv, trace_stack_pop as trace_stack_pop, trace_stack_push as trace_stack_push, trace_stack_top as trace_stack_top, trace_update_result as trace_update_result
from collections.abc import Sequence
from homeassistant import exceptions as exceptions
from homeassistant.components import device_automation as device_automation, scene as scene
from homeassistant.components.logger import LOGSEVERITY as LOGSEVERITY
from homeassistant.const import ATTR_AREA_ID as ATTR_AREA_ID, ATTR_DEVICE_ID as ATTR_DEVICE_ID, ATTR_ENTITY_ID as ATTR_ENTITY_ID, CONF_ALIAS as CONF_ALIAS, CONF_CHOOSE as CONF_CHOOSE, CONF_CONDITION as CONF_CONDITION, CONF_CONDITIONS as CONF_CONDITIONS, CONF_CONTINUE_ON_TIMEOUT as CONF_CONTINUE_ON_TIMEOUT, CONF_COUNT as CONF_COUNT, CONF_DEFAULT as CONF_DEFAULT, CONF_DELAY as CONF_DELAY, CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_DOMAIN as CONF_DOMAIN, CONF_EVENT as CONF_EVENT, CONF_EVENT_DATA as CONF_EVENT_DATA, CONF_EVENT_DATA_TEMPLATE as CONF_EVENT_DATA_TEMPLATE, CONF_MODE as CONF_MODE, CONF_REPEAT as CONF_REPEAT, CONF_SCENE as CONF_SCENE, CONF_SEQUENCE as CONF_SEQUENCE, CONF_SERVICE as CONF_SERVICE, CONF_TARGET as CONF_TARGET, CONF_TIMEOUT as CONF_TIMEOUT, CONF_UNTIL as CONF_UNTIL, CONF_VARIABLES as CONF_VARIABLES, CONF_WAIT_FOR_TRIGGER as CONF_WAIT_FOR_TRIGGER, CONF_WAIT_TEMPLATE as CONF_WAIT_TEMPLATE, CONF_WHILE as CONF_WHILE, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP, SERVICE_TURN_ON as SERVICE_TURN_ON
from homeassistant.core import Context as Context, HassJob as HassJob, HomeAssistant as HomeAssistant, SERVICE_CALL_LIMIT as SERVICE_CALL_LIMIT, callback as callback
from homeassistant.helpers import condition as condition, service as service, template as template
from homeassistant.helpers.condition import ConditionCheckerType as ConditionCheckerType, trace_condition_function as trace_condition_function
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect, async_dispatcher_send as async_dispatcher_send
from homeassistant.helpers.event import async_call_later as async_call_later, async_track_template as async_track_template
from homeassistant.helpers.script_variables import ScriptVariables as ScriptVariables
from homeassistant.helpers.trace import script_execution_set as script_execution_set
from homeassistant.helpers.trigger import async_initialize_triggers as async_initialize_triggers, async_validate_trigger_config as async_validate_trigger_config
from homeassistant.helpers.typing import ConfigType as ConfigType
from homeassistant.util import slugify as slugify
from homeassistant.util.dt import utcnow as utcnow
from types import MappingProxyType
from typing import Any, Callable, Dict, TypedDict, Union

SCRIPT_MODE_PARALLEL: str
SCRIPT_MODE_QUEUED: str
SCRIPT_MODE_RESTART: str
SCRIPT_MODE_SINGLE: str
SCRIPT_MODE_CHOICES: Any
DEFAULT_SCRIPT_MODE = SCRIPT_MODE_SINGLE
CONF_MAX: str
DEFAULT_MAX: int
CONF_MAX_EXCEEDED: str
_MAX_EXCEEDED_CHOICES: Any
DEFAULT_MAX_EXCEEDED: str
ATTR_CUR: str
ATTR_MAX: str
DATA_SCRIPTS: str
DATA_SCRIPT_BREAKPOINTS: str
RUN_ID_ANY: str
NODE_ANY: str
_LOGGER: Any
_LOG_EXCEPTION: Any
_TIMEOUT_MSG: str
_SHUTDOWN_MAX_WAIT: int
ACTION_TRACE_NODE_MAX_LEN: int
SCRIPT_BREAKPOINT_HIT: str
SCRIPT_DEBUG_CONTINUE_STOP: str
SCRIPT_DEBUG_CONTINUE_ALL: str

def action_trace_append(variables, path): ...
async def trace_action(hass, script_run, stop, variables) -> None: ...
def make_script_schema(schema, default_script_mode, extra=...): ...

STATIC_VALIDATION_ACTION_TYPES: Any

async def async_validate_actions_config(hass: HomeAssistant, actions: list[ConfigType]) -> list[ConfigType]: ...
async def async_validate_action_config(hass: HomeAssistant, config: ConfigType) -> ConfigType: ...

class _StopScript(Exception): ...

class _ScriptRun:
    _hass: Any
    _script: Any
    _variables: Any
    _context: Any
    _log_exceptions: Any
    _step: int
    _action: Any
    _stop: Any
    _stopped: Any
    def __init__(self, hass: HomeAssistant, script: Script, variables: dict[str, Any], context: Union[Context, None], log_exceptions: bool) -> None: ...
    def _changed(self) -> None: ...
    async def _async_get_condition(self, config): ...
    def _log(self, msg: str, *args: Any, level: int = ..., **kwargs: Any) -> None: ...
    def _step_log(self, default_message, timeout: Any | None = ...) -> None: ...
    async def async_run(self) -> None: ...
    async def _async_step(self, log_exceptions) -> None: ...
    def _finish(self) -> None: ...
    async def async_stop(self) -> None: ...
    def _log_exception(self, exception) -> None: ...
    def _get_pos_time_period_template(self, key): ...
    async def _async_delay_step(self) -> None: ...
    async def _async_wait_template_step(self) -> None: ...
    async def _async_run_long_action(self, long_task: asyncio.Task) -> None: ...
    async def _async_call_service_step(self) -> None: ...
    async def _async_device_step(self) -> None: ...
    async def _async_scene_step(self) -> None: ...
    async def _async_event_step(self) -> None: ...
    async def _async_condition_step(self) -> None: ...
    def _test_conditions(self, conditions, name, condition_path: Any | None = ...): ...
    async def _async_repeat_step(self) -> None: ...
    async def _async_choose_step(self) -> None: ...
    async def _async_wait_for_trigger_step(self) -> None: ...
    async def _async_variables_step(self) -> None: ...
    async def _async_run_script(self, script: Script) -> None: ...

class _QueuedScriptRun(_ScriptRun):
    lock_acquired: bool
    async def async_run(self) -> None: ...
    def _finish(self) -> None: ...

async def _async_stop_scripts_after_shutdown(hass, point_in_time) -> None: ...
async def _async_stop_scripts_at_shutdown(hass, event) -> None: ...
_VarsType = Union[Dict[str, Any], MappingProxyType]

def _referenced_extract_ids(data: dict[str, Any], key: str, found: set[str]) -> None: ...

class _ChooseData(TypedDict):
    choices: list[tuple[list[ConditionCheckerType], Script]]
    default: Union[Script, None]

class Script:
    _top_level: Any
    _hass: Any
    sequence: Any
    name: Any
    domain: Any
    running_description: Any
    _change_listener: Any
    _change_listener_job: Any
    script_mode: Any
    _log_exceptions: Any
    last_action: Any
    last_triggered: Any
    _runs: Any
    max_runs: Any
    _max_exceeded: Any
    _queue_lck: Any
    _config_cache: Any
    _repeat_script: Any
    _choose_data: Any
    _referenced_entities: Any
    _referenced_devices: Any
    _referenced_areas: Any
    variables: Any
    _variables_dynamic: Any
    def __init__(self, hass: HomeAssistant, sequence: Sequence[dict[str, Any]], name: str, domain: str, *, running_description: Union[str, None] = ..., change_listener: Union[Callable[..., Any], None] = ..., script_mode: str = ..., max_runs: int = ..., max_exceeded: str = ..., logger: Union[logging.Logger, None] = ..., log_exceptions: bool = ..., top_level: bool = ..., variables: Union[ScriptVariables, None] = ...) -> None: ...
    @property
    def change_listener(self) -> Union[Callable[..., Any], None]: ...
    @change_listener.setter
    def change_listener(self, change_listener: Callable[..., Any]) -> None: ...
    _logger: Any
    def _set_logger(self, logger: Union[logging.Logger, None] = ...) -> None: ...
    def update_logger(self, logger: Union[logging.Logger, None] = ...) -> None: ...
    def _changed(self) -> None: ...
    def _chain_change_listener(self, sub_script: Script) -> None: ...
    @property
    def is_running(self) -> bool: ...
    @property
    def runs(self) -> int: ...
    @property
    def supports_max(self) -> bool: ...
    @property
    def referenced_areas(self): ...
    @property
    def referenced_devices(self): ...
    @property
    def referenced_entities(self): ...
    def run(self, variables: Union[_VarsType, None] = ..., context: Union[Context, None] = ...) -> None: ...
    async def async_run(self, run_variables: Union[_VarsType, None] = ..., context: Union[Context, None] = ..., started_action: Union[Callable[..., Any], None] = ...) -> None: ...
    async def _async_stop(self, aws: list[asyncio.Task], update_state: bool, spare: Union[_ScriptRun, None]) -> None: ...
    async def async_stop(self, update_state: bool = ..., spare: Union[_ScriptRun, None] = ...) -> None: ...
    async def _async_get_condition(self, config): ...
    def _prep_repeat_script(self, step: int) -> Script: ...
    def _get_repeat_script(self, step: int) -> Script: ...
    async def _async_prep_choose_data(self, step: int) -> _ChooseData: ...
    async def _async_get_choose_data(self, step: int) -> _ChooseData: ...
    def _log(self, msg: str, *args: Any, level: int = ..., **kwargs: Any) -> None: ...

def breakpoint_clear(hass, key, run_id, node) -> None: ...
def breakpoint_clear_all(hass: HomeAssistant) -> None: ...
def breakpoint_set(hass, key, run_id, node) -> None: ...
def breakpoint_list(hass: HomeAssistant) -> list[dict[str, Any]]: ...
def debug_continue(hass, key, run_id) -> None: ...
def debug_step(hass, key, run_id) -> None: ...
def debug_stop(hass, key, run_id) -> None: ...



from collections.abc import Iterable
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers.typing import UNDEFINED as UNDEFINED, UndefinedType as UndefinedType
from homeassistant.loader import Integration as Integration, IntegrationNotFound as IntegrationNotFound, async_get_integration as async_get_integration
from typing import Any

PIP_TIMEOUT: int
DATA_PIP_LOCK: str
DATA_PKG_CACHE: str
DATA_INTEGRATIONS_WITH_REQS: str
CONSTRAINT_FILE: str
DISCOVERY_INTEGRATIONS: dict[str, Iterable[str]]

class RequirementsNotFound(HomeAssistantError):
    domain: Any
    requirements: Any
    def __init__(self, domain: str, requirements: list[str]) -> None: ...

async def async_get_integration_with_requirements(hass: HomeAssistant, domain: str, done: Union[set[str], None] = ...) -> Integration: ...
async def _async_process_integration(hass: HomeAssistant, integration: Integration, done: set[str]) -> None: ...
async def async_process_requirements(hass: HomeAssistant, name: str, requirements: list[str]) -> None: ...
def pip_kwargs(config_dir: Union[str, None]) -> dict[str, Any]: ...



from collections.abc import Awaitable, Iterable
from datetime import datetime, timedelta
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, ATTR_NOW as ATTR_NOW, EVENT_CORE_CONFIG_UPDATE as EVENT_CORE_CONFIG_UPDATE, EVENT_STATE_CHANGED as EVENT_STATE_CHANGED, EVENT_TIME_CHANGED as EVENT_TIME_CHANGED, MATCH_ALL as MATCH_ALL, SUN_EVENT_SUNRISE as SUN_EVENT_SUNRISE, SUN_EVENT_SUNSET as SUN_EVENT_SUNSET
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, Event as Event, HassJob as HassJob, HomeAssistant as HomeAssistant, State as State, callback as callback, split_entity_id as split_entity_id
from homeassistant.exceptions import TemplateError as TemplateError
from homeassistant.helpers.entity_registry import EVENT_ENTITY_REGISTRY_UPDATED as EVENT_ENTITY_REGISTRY_UPDATED
from homeassistant.helpers.ratelimit import KeyedRateLimit as KeyedRateLimit
from homeassistant.helpers.sun import get_astral_event_next as get_astral_event_next
from homeassistant.helpers.template import RenderInfo as RenderInfo, Template as Template, result_as_boolean as result_as_boolean
from homeassistant.helpers.typing import TemplateVarsType as TemplateVarsType
from homeassistant.loader import bind_hass as bind_hass
from homeassistant.util import dt as dt_util
from homeassistant.util.async_ import run_callback_threadsafe as run_callback_threadsafe
from typing import Any, Callable, List

TRACK_STATE_CHANGE_CALLBACKS: str
TRACK_STATE_CHANGE_LISTENER: str
TRACK_STATE_ADDED_DOMAIN_CALLBACKS: str
TRACK_STATE_ADDED_DOMAIN_LISTENER: str
TRACK_STATE_REMOVED_DOMAIN_CALLBACKS: str
TRACK_STATE_REMOVED_DOMAIN_LISTENER: str
TRACK_ENTITY_REGISTRY_UPDATED_CALLBACKS: str
TRACK_ENTITY_REGISTRY_UPDATED_LISTENER: str
_ALL_LISTENER: str
_DOMAINS_LISTENER: str
_ENTITIES_LISTENER: str
_LOGGER: Any

class TrackStates:
    all_states: bool
    entities: set
    domains: set

class TrackTemplate:
    template: Template
    variables: TemplateVarsType
    rate_limit: Union[timedelta, None]

class TrackTemplateResult:
    template: Template
    last_result: Any
    result: Any

def threaded_listener_factory(async_factory: Callable[..., Any]) -> Callable[..., CALLBACK_TYPE]: ...
def async_track_state_change(hass: HomeAssistant, entity_ids: Union[str, Iterable[str]], action: Callable[[str, State, State], Union[Awaitable[None], None]], from_state: Union[None, str, Iterable[str]] = ..., to_state: Union[None, str, Iterable[str]] = ...) -> CALLBACK_TYPE: ...

track_state_change: Any

def async_track_state_change_event(hass: HomeAssistant, entity_ids: Union[str, Iterable[str]], action: Callable[[Event], Any]) -> Callable[[], None]: ...
def _remove_empty_listener() -> None: ...
def _async_remove_indexed_listeners(hass: HomeAssistant, data_key: str, listener_key: str, storage_keys: Iterable[str], job: HassJob) -> None: ...
def async_track_entity_registry_updated_event(hass: HomeAssistant, entity_ids: Union[str, Iterable[str]], action: Callable[[Event], Any]) -> Callable[[], None]: ...
def _async_dispatch_domain_event(hass: HomeAssistant, event: Event, callbacks: dict[str, list]) -> None: ...
def async_track_state_added_domain(hass: HomeAssistant, domains: Union[str, Iterable[str]], action: Callable[[Event], Any]) -> Callable[[], None]: ...
def async_track_state_removed_domain(hass: HomeAssistant, domains: Union[str, Iterable[str]], action: Callable[[Event], Any]) -> Callable[[], None]: ...
def _async_string_to_lower_list(instr: Union[str, Iterable[str]]) -> list[str]: ...

class _TrackStateChangeFiltered:
    hass: Any
    _action: Any
    _listeners: Any
    _last_track_states: Any
    def __init__(self, hass: HomeAssistant, track_states: TrackStates, action: Callable[[Event], Any]) -> None: ...
    def async_setup(self) -> None: ...
    @property
    def listeners(self) -> dict: ...
    def async_update_listeners(self, new_track_states: TrackStates) -> None: ...
    def async_remove(self) -> None: ...
    def _cancel_listener(self, listener_name: str) -> None: ...
    def _setup_entities_listener(self, domains: set, entities: set) -> None: ...
    def _setup_domains_listener(self, domains: set) -> None: ...
    def _setup_all_listener(self) -> None: ...

def async_track_state_change_filtered(hass: HomeAssistant, track_states: TrackStates, action: Callable[[Event], Any]) -> _TrackStateChangeFiltered: ...
def async_track_template(hass: HomeAssistant, template: Template, action: Callable[[str, Union[State, None], Union[State, None]], Union[Awaitable[None], None]], variables: Union[TemplateVarsType, None] = ...) -> Callable[[], None]: ...

track_template: Any

class _TrackTemplateResultInfo:
    hass: Any
    _job: Any
    _track_templates: Any
    _last_result: Any
    _rate_limit: Any
    _info: Any
    _track_state_changes: Any
    _time_listeners: Any
    def __init__(self, hass: HomeAssistant, track_templates: Iterable[TrackTemplate], action: Callable) -> None: ...
    def async_setup(self, raise_on_template_error: bool, strict: bool = ...) -> None: ...
    @property
    def listeners(self) -> dict: ...
    def _setup_time_listener(self, template: Template, has_time: bool) -> None: ...
    def _update_time_listeners(self) -> None: ...
    def async_remove(self) -> None: ...
    def async_refresh(self) -> None: ...
    def _render_template_if_ready(self, track_template_: TrackTemplate, now: datetime, event: Union[Event, None]) -> Union[bool, TrackTemplateResult]: ...
    def _refresh(self, event: Union[Event, None], track_templates: Union[Iterable[TrackTemplate], None] = ..., replayed: Union[bool, None] = ...) -> None: ...
TrackTemplateResultListener = Callable[[Event, List[TrackTemplateResult]], None]

def async_track_template_result(hass: HomeAssistant, track_templates: Iterable[TrackTemplate], action: TrackTemplateResultListener, raise_on_template_error: bool = ..., strict: bool = ...) -> _TrackTemplateResultInfo: ...
def async_track_same_state(hass: HomeAssistant, period: timedelta, action: Callable[..., Union[Awaitable[None], None]], async_check_same_func: Callable[[str, Union[State, None], Union[State, None]], bool], entity_ids: Union[str, Iterable[str]] = ...) -> CALLBACK_TYPE: ...

track_same_state: Any

def async_track_point_in_time(hass: HomeAssistant, action: Union[HassJob, Callable[..., Union[Awaitable[None], None]]], point_in_time: datetime) -> CALLBACK_TYPE: ...

track_point_in_time: Any

def async_track_point_in_utc_time(hass: HomeAssistant, action: Union[HassJob, Callable[..., Union[Awaitable[None], None]]], point_in_time: datetime) -> CALLBACK_TYPE: ...

track_point_in_utc_time: Any

def async_call_later(hass: HomeAssistant, delay: Union[float, timedelta], action: Union[HassJob, Callable[..., Union[Awaitable[None], None]]]) -> CALLBACK_TYPE: ...

call_later: Any

def async_track_time_interval(hass: HomeAssistant, action: Callable[..., Union[Awaitable[None], None]], interval: timedelta) -> CALLBACK_TYPE: ...

track_time_interval: Any

class SunListener:
    hass: HomeAssistant
    job: HassJob
    event: str
    offset: Union[timedelta, None]
    _unsub_sun: Union[CALLBACK_TYPE, None]
    _unsub_config: Union[CALLBACK_TYPE, None]
    def async_attach(self) -> None: ...
    def async_detach(self) -> None: ...
    def _listen_next_sun_event(self) -> None: ...
    def _handle_sun_event(self, _now: Any) -> None: ...
    def _handle_config_event(self, _event: Any) -> None: ...
    def __init__(self, hass, job, event, offset, unsub_sun, unsub_config) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

def async_track_sunrise(hass: HomeAssistant, action: Callable[..., None], offset: Union[timedelta, None] = ...) -> CALLBACK_TYPE: ...

track_sunrise: Any

def async_track_sunset(hass: HomeAssistant, action: Callable[..., None], offset: Union[timedelta, None] = ...) -> CALLBACK_TYPE: ...

track_sunset: Any
time_tracker_utcnow = dt_util.utcnow

def async_track_utc_time_change(hass: HomeAssistant, action: Callable[..., Union[Awaitable[None], None]], hour: Union[Any, None] = ..., minute: Union[Any, None] = ..., second: Union[Any, None] = ..., local: bool = ...) -> CALLBACK_TYPE: ...

track_utc_time_change: Any

def async_track_time_change(hass: HomeAssistant, action: Callable[..., None], hour: Union[Any, None] = ..., minute: Union[Any, None] = ..., second: Union[Any, None] = ...) -> CALLBACK_TYPE: ...

track_time_change: Any

def process_state_match(parameter: Union[None, str, Iterable[str]]) -> Callable[[Union[str, None]], bool]: ...
def _entities_domains_from_render_infos(render_infos: Iterable[RenderInfo]) -> tuple[set[str], set[str]]: ...
def _render_infos_needs_all_listener(render_infos: Iterable[RenderInfo]) -> bool: ...
def _render_infos_to_track_states(render_infos: Iterable[RenderInfo]) -> TrackStates: ...
def _event_triggers_rerender(event: Event, info: RenderInfo) -> bool: ...
def _rate_limit_for_event(event: Event, info: RenderInfo, track_template_: TrackTemplate) -> Union[timedelta, None]: ...
def _suppress_domain_all_in_render_info(render_info: RenderInfo) -> RenderInfo: ...



from . import FritzBoxEntity as FritzBoxEntity
from .const import ATTR_STATE_DEVICE_LOCKED as ATTR_STATE_DEVICE_LOCKED, ATTR_STATE_LOCKED as ATTR_STATE_LOCKED, CONF_COORDINATOR as CONF_COORDINATOR
from .model import SensorExtraAttributes as SensorExtraAttributes
from datetime import datetime
from homeassistant.components.sensor import ATTR_STATE_CLASS as ATTR_STATE_CLASS, STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT, SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_DEVICE_CLASS as ATTR_DEVICE_CLASS, ATTR_ENTITY_ID as ATTR_ENTITY_ID, ATTR_NAME as ATTR_NAME, ATTR_UNIT_OF_MEASUREMENT as ATTR_UNIT_OF_MEASUREMENT, DEVICE_CLASS_BATTERY as DEVICE_CLASS_BATTERY, DEVICE_CLASS_ENERGY as DEVICE_CLASS_ENERGY, DEVICE_CLASS_POWER as DEVICE_CLASS_POWER, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, ENERGY_KILO_WATT_HOUR as ENERGY_KILO_WATT_HOUR, PERCENTAGE as PERCENTAGE, POWER_WATT as POWER_WATT, TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.util.dt import utc_from_timestamp as utc_from_timestamp

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class FritzBoxBatterySensor(FritzBoxEntity, SensorEntity):
    @property
    def state(self) -> Union[int, None]: ...

class FritzBoxPowerSensor(FritzBoxEntity, SensorEntity):
    @property
    def state(self) -> Union[float, None]: ...

class FritzBoxEnergySensor(FritzBoxEntity, SensorEntity):
    @property
    def state(self) -> Union[float, None]: ...
    @property
    def last_reset(self) -> datetime: ...

class FritzBoxTempSensor(FritzBoxEntity, SensorEntity):
    @property
    def state(self) -> Union[float, None]: ...
    @property
    def extra_state_attributes(self) -> SensorExtraAttributes: ...



from aiohttp import web
from ssl import SSLContext
from typing import Any

class HomeAssistantTCPSite(web.BaseSite):
    __slots__: Any
    _host: Any
    _port: Any
    _reuse_address: Any
    _reuse_port: Any
    def __init__(self, runner: web.BaseRunner, host: Union[None, str, list[str]], port: int, *, shutdown_timeout: float = ..., ssl_context: Union[SSLContext, None] = ..., backlog: int = ..., reuse_address: Union[bool, None] = ..., reuse_port: Union[bool, None] = ...) -> None: ...
    @property
    def name(self) -> str: ...
    _server: Any
    async def start(self) -> None: ...



from typing import Any

DOMAIN: str
LOGGER: Any
API_SENSOR_PAIRED_SENSOR_STATUS: str
API_SENSOR_PAIR_DUMP: str
API_SYSTEM_DIAGNOSTICS: str
API_SYSTEM_ONBOARD_SENSOR_STATUS: str
API_VALVE_STATUS: str
API_WIFI_STATUS: str
CONF_UID: str
DATA_CLIENT: str
DATA_COORDINATOR: str
DATA_COORDINATOR_PAIRED_SENSOR: str
DATA_PAIRED_SENSOR_MANAGER: str
DATA_UNSUB_DISPATCHER_CONNECT: str
SIGNAL_PAIRED_SENSOR_COORDINATOR_ADDED: str



from . import BrotherDataUpdateCoordinator as BrotherDataUpdateCoordinator
from .const import ATTRS_MAP as ATTRS_MAP, ATTR_COUNTER as ATTR_COUNTER, ATTR_MANUFACTURER as ATTR_MANUFACTURER, ATTR_REMAINING_PAGES as ATTR_REMAINING_PAGES, ATTR_UPTIME as ATTR_UPTIME, DATA_CONFIG_ENTRY as DATA_CONFIG_ENTRY, DOMAIN as DOMAIN, SENSOR_TYPES as SENSOR_TYPES
from homeassistant.components.sensor import SensorEntity as SensorEntity, SensorEntityDescription as SensorEntityDescription
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import StateType as StateType
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity
from typing import Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class BrotherPrinterSensor(CoordinatorEntity, SensorEntity):
    _attrs: Any
    _attr_device_info: Any
    _attr_name: Any
    _attr_unique_id: Any
    entity_description: Any
    def __init__(self, coordinator: BrotherDataUpdateCoordinator, description: SensorEntityDescription, device_info: DeviceInfo) -> None: ...
    @property
    def state(self) -> StateType: ...
    @property
    def extra_state_attributes(self) -> dict[str, Any]: ...



from .model import AirlySensorEntityDescription as AirlySensorEntityDescription
from homeassistant.components.sensor import STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT
from homeassistant.const import CONCENTRATION_MICROGRAMS_PER_CUBIC_METER as CONCENTRATION_MICROGRAMS_PER_CUBIC_METER, DEVICE_CLASS_HUMIDITY as DEVICE_CLASS_HUMIDITY, DEVICE_CLASS_PRESSURE as DEVICE_CLASS_PRESSURE, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, PERCENTAGE as PERCENTAGE, PRESSURE_HPA as PRESSURE_HPA, TEMP_CELSIUS as TEMP_CELSIUS
from typing import Final

ATTR_API_ADVICE: Final[str]
ATTR_API_CAQI: Final[str]
ATTR_API_CAQI_DESCRIPTION: Final[str]
ATTR_API_CAQI_LEVEL: Final[str]
ATTR_API_HUMIDITY: Final[str]
ATTR_API_PM10: Final[str]
ATTR_API_PM1: Final[str]
ATTR_API_PM25: Final[str]
ATTR_API_PRESSURE: Final[str]
ATTR_API_TEMPERATURE: Final[str]
ATTR_ADVICE: Final[str]
ATTR_DESCRIPTION: Final[str]
ATTR_LEVEL: Final[str]
ATTR_LIMIT: Final[str]
ATTR_PERCENT: Final[str]
SUFFIX_PERCENT: Final[str]
SUFFIX_LIMIT: Final[str]
ATTRIBUTION: Final[str]
CONF_USE_NEAREST: Final[str]
DEFAULT_NAME: Final[str]
DOMAIN: Final[str]
LABEL_ADVICE: Final[str]
MANUFACTURER: Final[str]
MAX_UPDATE_INTERVAL: Final[int]
MIN_UPDATE_INTERVAL: Final[int]
NO_AIRLY_SENSORS: Final[str]
SENSOR_TYPES: tuple[AirlySensorEntityDescription, ...]



from . import DOMAIN as DOMAIN
from homeassistant.components.device_automation import toggle_entity as toggle_entity
from homeassistant.const import CONF_DOMAIN as CONF_DOMAIN
from homeassistant.core import Context as Context, HomeAssistant as HomeAssistant
from homeassistant.helpers.typing import ConfigType as ConfigType, TemplateVarsType as TemplateVarsType
from typing import Any

ACTION_SCHEMA: Any

async def async_call_action_from_config(hass: HomeAssistant, config: ConfigType, variables: TemplateVarsType, context: Context) -> None: ...
async def async_get_actions(hass: HomeAssistant, device_id: str) -> list[dict]: ...



from .const import ATTR_ALARM_DURATION as ATTR_ALARM_DURATION, ATTR_ALARM_VOLUME as ATTR_ALARM_VOLUME, ATTR_CHIME_VOLUME as ATTR_CHIME_VOLUME, ATTR_ENTRY_DELAY_AWAY as ATTR_ENTRY_DELAY_AWAY, ATTR_ENTRY_DELAY_HOME as ATTR_ENTRY_DELAY_HOME, ATTR_EXIT_DELAY_AWAY as ATTR_EXIT_DELAY_AWAY, ATTR_EXIT_DELAY_HOME as ATTR_EXIT_DELAY_HOME, ATTR_LIGHT as ATTR_LIGHT, ATTR_VOICE_PROMPT_VOLUME as ATTR_VOICE_PROMPT_VOLUME, DATA_CLIENT as DATA_CLIENT, DEFAULT_SCAN_INTERVAL as DEFAULT_SCAN_INTERVAL, DOMAIN as DOMAIN, LOGGER as LOGGER, VOLUMES as VOLUMES
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_CODE as ATTR_CODE, CONF_CODE as CONF_CODE, CONF_PASSWORD as CONF_PASSWORD, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import CoreState as CoreState, HomeAssistant as HomeAssistant, ServiceCall as ServiceCall, callback as callback
from homeassistant.exceptions import ConfigEntryAuthFailed as ConfigEntryAuthFailed, ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import aiohttp_client as aiohttp_client
from homeassistant.helpers.service import async_register_admin_service as async_register_admin_service, verify_domain_control as verify_domain_control
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity, DataUpdateCoordinator as DataUpdateCoordinator, UpdateFailed as UpdateFailed
from simplipy.api import API as API
from simplipy.sensor.v2 import SensorV2 as SensorV2
from simplipy.sensor.v3 import SensorV3 as SensorV3
from simplipy.system import SystemNotification as SystemNotification
from simplipy.system.v2 import SystemV2 as SystemV2
from simplipy.system.v3 import SystemV3
from typing import Any

EVENT_SIMPLISAFE_NOTIFICATION: str
DEFAULT_SOCKET_MIN_RETRY: int
PLATFORMS: Any
ATTR_CATEGORY: str
ATTR_MESSAGE: str
ATTR_PIN_LABEL: str
ATTR_PIN_LABEL_OR_VALUE: str
ATTR_PIN_VALUE: str
ATTR_SYSTEM_ID: str
ATTR_TIMESTAMP: str
SERVICE_BASE_SCHEMA: Any
SERVICE_REMOVE_PIN_SCHEMA: Any
SERVICE_SET_PIN_SCHEMA: Any
SERVICE_SET_SYSTEM_PROPERTIES_SCHEMA: Any
CONFIG_SCHEMA: Any

async def async_get_client_id(hass: HomeAssistant) -> str: ...
async def async_register_base_station(hass: HomeAssistant, system: Union[SystemV2, SystemV3], config_entry_id: str) -> None: ...
def _async_standardize_config_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> None: ...
async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_reload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> None: ...

class SimpliSafe:
    _api: Any
    _hass: Any
    _system_notifications: Any
    config_entry: Any
    coordinator: Any
    systems: Any
    def __init__(self, hass: HomeAssistant, config_entry: ConfigEntry, api: API) -> None: ...
    def _async_process_new_notifications(self, system: Union[SystemV2, SystemV3]) -> None: ...
    async def async_init(self) -> None: ...
    async def async_update(self) -> None: ...

class SimpliSafeEntity(CoordinatorEntity):
    _serial: Any
    _attr_extra_state_attributes: Any
    _attr_device_info: Any
    _attr_name: Any
    _attr_unique_id: Any
    _online: bool
    _simplisafe: Any
    _system: Any
    def __init__(self, simplisafe: SimpliSafe, system: Union[SystemV2, SystemV3], name: str, *, serial: Union[str, None] = ...) -> None: ...
    @property
    def available(self) -> bool: ...
    def _handle_coordinator_update(self) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    def async_update_from_rest_api(self) -> None: ...

class SimpliSafeBaseSensor(SimpliSafeEntity):
    _attr_device_info: Any
    _attr_name: Any
    _sensor: Any
    def __init__(self, simplisafe: SimpliSafe, system: Union[SystemV2, SystemV3], sensor: Union[SensorV2, SensorV3]) -> None: ...



from . import OpenUV as OpenUV, OpenUvEntity as OpenUvEntity
from .const import DATA_CLIENT as DATA_CLIENT, DATA_PROTECTION_WINDOW as DATA_PROTECTION_WINDOW, DOMAIN as DOMAIN, LOGGER as LOGGER, TYPE_PROTECTION_WINDOW as TYPE_PROTECTION_WINDOW
from homeassistant.components.binary_sensor import BinarySensorEntity as BinarySensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.util.dt import as_local as as_local, parse_datetime as parse_datetime, utcnow as utcnow
from typing import Any

ATTR_PROTECTION_WINDOW_ENDING_TIME: str
ATTR_PROTECTION_WINDOW_ENDING_UV: str
ATTR_PROTECTION_WINDOW_STARTING_TIME: str
ATTR_PROTECTION_WINDOW_STARTING_UV: str
BINARY_SENSORS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class OpenUvBinarySensor(OpenUvEntity, BinarySensorEntity):
    _attr_icon: Any
    _attr_name: Any
    def __init__(self, openuv: OpenUV, sensor_type: str, name: str, icon: str) -> None: ...
    _attr_available: bool
    _attr_is_on: bool
    def update_from_latest_data(self) -> None: ...



from .const import CATEGORY_CDC_REPORT as CATEGORY_CDC_REPORT, CATEGORY_USER_REPORT as CATEGORY_USER_REPORT, DATA_COORDINATOR as DATA_COORDINATOR, DOMAIN as DOMAIN, LOGGER as LOGGER
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_LATITUDE as CONF_LATITUDE, CONF_LONGITUDE as CONF_LONGITUDE
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers import aiohttp_client as aiohttp_client
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator, UpdateFailed as UpdateFailed
from typing import Any

DEFAULT_UPDATE_INTERVAL: Any
CONFIG_SCHEMA: Any
PLATFORMS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...



from .const import DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN
from .discovery import ZwaveDiscoveryInfo as ZwaveDiscoveryInfo
from .entity import ZWaveBaseEntity as ZWaveBaseEntity
from homeassistant.components.switch import SwitchEntity as SwitchEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any
from zwave_js_server.client import Client as ZwaveClient

LOGGER: Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class ZWaveSwitch(ZWaveBaseEntity, SwitchEntity):
    @property
    def is_on(self) -> Union[bool, None]: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...

class ZWaveBarrierEventSignalingSwitch(ZWaveBaseEntity, SwitchEntity):
    _state: Any
    _attr_name: Any
    def __init__(self, config_entry: ConfigEntry, client: ZwaveClient, info: ZwaveDiscoveryInfo) -> None: ...
    def on_value_update(self) -> None: ...
    @property
    def is_on(self) -> Union[bool, None]: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    def _update_state(self) -> None: ...



from . import DOMAIN as DOMAIN, SUPPORT_CLOSE as SUPPORT_CLOSE, SUPPORT_OPEN as SUPPORT_OPEN, SUPPORT_SET_POSITION as SUPPORT_SET_POSITION, SUPPORT_SET_TILT_POSITION as SUPPORT_SET_TILT_POSITION
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, CONF_ABOVE as CONF_ABOVE, CONF_BELOW as CONF_BELOW, CONF_CONDITION as CONF_CONDITION, CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_TYPE as CONF_TYPE, STATE_CLOSED as STATE_CLOSED, STATE_CLOSING as STATE_CLOSING, STATE_OPEN as STATE_OPEN, STATE_OPENING as STATE_OPENING
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import condition as condition, entity_registry as entity_registry, template as template
from homeassistant.helpers.config_validation import DEVICE_CONDITION_BASE_SCHEMA as DEVICE_CONDITION_BASE_SCHEMA
from homeassistant.helpers.entity import get_supported_features as get_supported_features
from homeassistant.helpers.typing import ConfigType as ConfigType, TemplateVarsType as TemplateVarsType
from typing import Any

POSITION_CONDITION_TYPES: Any
STATE_CONDITION_TYPES: Any
POSITION_CONDITION_SCHEMA: Any
STATE_CONDITION_SCHEMA: Any
CONDITION_SCHEMA: Any

async def async_get_conditions(hass: HomeAssistant, device_id: str) -> list[dict]: ...
async def async_get_condition_capabilities(hass: HomeAssistant, config: dict) -> dict: ...
def async_condition_from_config(config: ConfigType, config_validation: bool) -> condition.ConditionCheckerType: ...



from . import SynoApi as SynoApi, SynologyDSMBaseEntity as SynologyDSMBaseEntity, SynologyDSMDeviceEntity as SynologyDSMDeviceEntity
from .const import COORDINATOR_CENTRAL as COORDINATOR_CENTRAL, DOMAIN as DOMAIN, SECURITY_BINARY_SENSORS as SECURITY_BINARY_SENSORS, STORAGE_DISK_BINARY_SENSORS as STORAGE_DISK_BINARY_SENSORS, SYNO_API as SYNO_API, UPGRADE_BINARY_SENSORS as UPGRADE_BINARY_SENSORS
from homeassistant.components.binary_sensor import BinarySensorEntity as BinarySensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_DISKS as CONF_DISKS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class SynoDSMSecurityBinarySensor(SynologyDSMBaseEntity, BinarySensorEntity):
    @property
    def is_on(self) -> bool: ...
    @property
    def available(self) -> bool: ...
    @property
    def extra_state_attributes(self) -> dict[str, str]: ...

class SynoDSMStorageBinarySensor(SynologyDSMDeviceEntity, BinarySensorEntity):
    @property
    def is_on(self) -> bool: ...

class SynoDSMUpgradeBinarySensor(SynologyDSMBaseEntity, BinarySensorEntity):
    @property
    def is_on(self) -> bool: ...
    @property
    def available(self) -> bool: ...



from . import CONF_RETAIN as CONF_RETAIN, CONF_VERSION as CONF_VERSION, DEFAULT_VERSION as DEFAULT_VERSION
from .const import CONF_BAUD_RATE as CONF_BAUD_RATE, CONF_GATEWAY_TYPE as CONF_GATEWAY_TYPE, CONF_GATEWAY_TYPE_ALL as CONF_GATEWAY_TYPE_ALL, CONF_GATEWAY_TYPE_MQTT as CONF_GATEWAY_TYPE_MQTT, CONF_GATEWAY_TYPE_SERIAL as CONF_GATEWAY_TYPE_SERIAL, CONF_GATEWAY_TYPE_TCP as CONF_GATEWAY_TYPE_TCP, CONF_PERSISTENCE_FILE as CONF_PERSISTENCE_FILE, CONF_TCP_PORT as CONF_TCP_PORT, CONF_TOPIC_IN_PREFIX as CONF_TOPIC_IN_PREFIX, CONF_TOPIC_OUT_PREFIX as CONF_TOPIC_OUT_PREFIX, ConfGatewayType as ConfGatewayType, DOMAIN as DOMAIN
from .gateway import MQTT_COMPONENT as MQTT_COMPONENT, is_serial_port as is_serial_port, is_socket_address as is_socket_address, try_connect as try_connect
from homeassistant import config_entries as config_entries
from homeassistant.components.mqtt import valid_publish_topic as valid_publish_topic, valid_subscribe_topic as valid_subscribe_topic
from homeassistant.components.mysensors import CONF_DEVICE as CONF_DEVICE, DEFAULT_BAUD_RATE as DEFAULT_BAUD_RATE, DEFAULT_TCP_PORT as DEFAULT_TCP_PORT, is_persistence_file as is_persistence_file
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from typing import Any

_LOGGER: Any

def _get_schema_common(user_input: dict[str, str]) -> dict: ...
def _validate_version(version: str) -> dict[str, str]: ...
def _is_same_device(gw_type: ConfGatewayType, user_input: dict[str, Any], entry: ConfigEntry) -> bool: ...

class MySensorsConfigFlowHandler(config_entries.ConfigFlow):
    _gw_type: Any
    def __init__(self) -> None: ...
    async def async_step_import(self, user_input: dict[str, Any]) -> FlowResult: ...
    async def async_step_user(self, user_input: Union[dict[str, str], None] = ...) -> FlowResult: ...
    async def async_step_gw_serial(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_gw_tcp(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    def _check_topic_exists(self, topic: str) -> bool: ...
    async def async_step_gw_mqtt(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    def _async_create_entry(self, user_input: dict[str, Any]) -> FlowResult: ...
    def _normalize_persistence_file(self, path: str) -> str: ...
    async def validate_common(self, gw_type: ConfGatewayType, errors: dict[str, str], user_input: dict[str, Any]) -> dict[str, str]: ...



import logging
from typing import Final

ATTR_STATE_BATTERY_LOW: Final[str]
ATTR_STATE_DEVICE_LOCKED: Final[str]
ATTR_STATE_HOLIDAY_MODE: Final[str]
ATTR_STATE_LOCKED: Final[str]
ATTR_STATE_SUMMER_MODE: Final[str]
ATTR_STATE_WINDOW_OPEN: Final[str]
ATTR_TEMPERATURE_UNIT: Final[str]
CONF_CONNECTIONS: Final[str]
CONF_COORDINATOR: Final[str]
DEFAULT_HOST: Final[str]
DEFAULT_USERNAME: Final[str]
DOMAIN: Final[str]
LOGGER: Final[logging.Logger]
PLATFORMS: Final[list[str]]



import re
import voluptuous as vol
from collections.abc import Hashable
from datetime import date as date_sys, datetime as datetime_sys, time as time_sys, timedelta
from enum import Enum
from homeassistant.const import ATTR_AREA_ID as ATTR_AREA_ID, ATTR_DEVICE_ID as ATTR_DEVICE_ID, ATTR_ENTITY_ID as ATTR_ENTITY_ID, CONF_ABOVE as CONF_ABOVE, CONF_ALIAS as CONF_ALIAS, CONF_ATTRIBUTE as CONF_ATTRIBUTE, CONF_BELOW as CONF_BELOW, CONF_CHOOSE as CONF_CHOOSE, CONF_CONDITION as CONF_CONDITION, CONF_CONDITIONS as CONF_CONDITIONS, CONF_CONTINUE_ON_TIMEOUT as CONF_CONTINUE_ON_TIMEOUT, CONF_COUNT as CONF_COUNT, CONF_DEFAULT as CONF_DEFAULT, CONF_DELAY as CONF_DELAY, CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_ENTITY_NAMESPACE as CONF_ENTITY_NAMESPACE, CONF_EVENT as CONF_EVENT, CONF_EVENT_DATA as CONF_EVENT_DATA, CONF_EVENT_DATA_TEMPLATE as CONF_EVENT_DATA_TEMPLATE, CONF_FOR as CONF_FOR, CONF_ID as CONF_ID, CONF_PLATFORM as CONF_PLATFORM, CONF_REPEAT as CONF_REPEAT, CONF_SCAN_INTERVAL as CONF_SCAN_INTERVAL, CONF_SCENE as CONF_SCENE, CONF_SEQUENCE as CONF_SEQUENCE, CONF_SERVICE as CONF_SERVICE, CONF_SERVICE_TEMPLATE as CONF_SERVICE_TEMPLATE, CONF_STATE as CONF_STATE, CONF_TARGET as CONF_TARGET, CONF_TIMEOUT as CONF_TIMEOUT, CONF_UNIT_SYSTEM_IMPERIAL as CONF_UNIT_SYSTEM_IMPERIAL, CONF_UNIT_SYSTEM_METRIC as CONF_UNIT_SYSTEM_METRIC, CONF_UNTIL as CONF_UNTIL, CONF_VALUE_TEMPLATE as CONF_VALUE_TEMPLATE, CONF_VARIABLES as CONF_VARIABLES, CONF_WAIT_FOR_TRIGGER as CONF_WAIT_FOR_TRIGGER, CONF_WAIT_TEMPLATE as CONF_WAIT_TEMPLATE, CONF_WHILE as CONF_WHILE, ENTITY_MATCH_ALL as ENTITY_MATCH_ALL, ENTITY_MATCH_NONE as ENTITY_MATCH_NONE, SUN_EVENT_SUNRISE as SUN_EVENT_SUNRISE, SUN_EVENT_SUNSET as SUN_EVENT_SUNSET, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT, WEEKDAYS as WEEKDAYS
from homeassistant.core import split_entity_id as split_entity_id, valid_entity_id as valid_entity_id
from homeassistant.exceptions import TemplateError as TemplateError
from homeassistant.helpers import template as template_helper
from homeassistant.helpers.logging import KeywordStyleAdapter as KeywordStyleAdapter
from homeassistant.util import raise_if_invalid_path as raise_if_invalid_path
from typing import Any, Callable, TypeVar

TIME_PERIOD_ERROR: str
byte: Any
small_float: Any
positive_int: Any
positive_float: Any
latitude: Any
longitude: Any
gps: Any
sun_event: Any
port: Any
T = TypeVar('T')

def path(value: Any) -> str: ...
def has_at_least_one_key(*keys: str) -> Callable: ...
def has_at_most_one_key(*keys: str) -> Callable[[dict], dict]: ...
def boolean(value: Any) -> bool: ...

_WS: Any

def whitespace(value: Any) -> str: ...
def isdevice(value: Any) -> str: ...
def matches_regex(regex: str) -> Callable[[Any], str]: ...
def is_regex(value: Any) -> re.Pattern[Any]: ...
def isfile(value: Any) -> str: ...
def isdir(value: Any) -> str: ...
def ensure_list(value: Union[T, list[T], None]) -> list[T]: ...
def entity_id(value: Any) -> str: ...
def entity_ids(value: Union[str, list]) -> list[str]: ...

comp_entity_ids: Any

def entity_domain(domain: Union[str, list[str]]) -> Callable[[Any], str]: ...
def entities_domain(domain: Union[str, list[str]]) -> Callable[[Union[str, list]], list[str]]: ...
def enum(enumClass: type[Enum]) -> vol.All: ...
def icon(value: Any) -> str: ...

time_period_dict: Any

def time(value: Any) -> time_sys: ...
def date(value: Any) -> date_sys: ...
def time_period_str(value: str) -> timedelta: ...
def time_period_seconds(value: Union[float, str]) -> timedelta: ...

time_period: Any

def match_all(value: T) -> T: ...
def positive_timedelta(value: timedelta) -> timedelta: ...

positive_time_period_dict: Any
positive_time_period: Any

def remove_falsy(value: list[T]) -> list[T]: ...
def service(value: Any) -> str: ...
def slug(value: Any) -> str: ...
def schema_with_slug_keys(value_schema: Union[T, Callable], *, slug_validator: Callable[[Any], str] = ...) -> Callable: ...
def slugify(value: Any) -> str: ...
def string(value: Any) -> str: ...
def string_with_no_html(value: Any) -> str: ...
def temperature_unit(value: Any) -> str: ...

unit_system: Any

def template(value: Union[Any, None]) -> template_helper.Template: ...
def dynamic_template(value: Union[Any, None]) -> template_helper.Template: ...
def template_complex(value: Any) -> Any: ...

positive_time_period_template: Any

def datetime(value: Any) -> datetime_sys: ...
def time_zone(value: str) -> str: ...

weekdays: Any

def socket_timeout(value: Union[Any, None]) -> object: ...
def url(value: Any) -> str: ...
def x10_address(value: str) -> str: ...
def uuid4_hex(value: Any) -> str: ...
def ensure_list_csv(value: Any) -> list: ...

class multi_select:
    options: Any
    def __init__(self, options: dict) -> None: ...
    def __call__(self, selected: list) -> list: ...

def deprecated(key: str, replacement_key: Union[str, None] = ..., default: Union[Any, None] = ...) -> Callable[[dict], dict]: ...
def key_value_schemas(key: str, value_schemas: dict[Hashable, vol.Schema]) -> Callable[[Any], dict[Hashable, Any]]: ...
def key_dependency(key: Hashable, dependency: Hashable) -> Callable[[dict[Hashable, Any]], dict[Hashable, Any]]: ...
def custom_serializer(schema: Any) -> Any: ...

PLATFORM_SCHEMA: Any
PLATFORM_SCHEMA_BASE: Any
ENTITY_SERVICE_FIELDS: Any

def make_entity_service_schema(schema: dict, *, extra: int = ...) -> vol.Schema: ...

SCRIPT_VARIABLES_SCHEMA: Any

def script_action(value: Any) -> dict: ...

SCRIPT_SCHEMA: Any
SCRIPT_ACTION_BASE_SCHEMA: Any
EVENT_SCHEMA: Any
SERVICE_SCHEMA: Any
NUMERIC_STATE_THRESHOLD_SCHEMA: Any
CONDITION_BASE_SCHEMA: Any
NUMERIC_STATE_CONDITION_SCHEMA: Any
STATE_CONDITION_BASE_SCHEMA: Any
STATE_CONDITION_STATE_SCHEMA: Any
STATE_CONDITION_ATTRIBUTE_SCHEMA: Any

def STATE_CONDITION_SCHEMA(value: Any) -> dict: ...

SUN_CONDITION_SCHEMA: Any
TEMPLATE_CONDITION_SCHEMA: Any
TIME_CONDITION_SCHEMA: Any
TRIGGER_CONDITION_SCHEMA: Any
ZONE_CONDITION_SCHEMA: Any
AND_CONDITION_SCHEMA: Any
OR_CONDITION_SCHEMA: Any
NOT_CONDITION_SCHEMA: Any
DEVICE_CONDITION_BASE_SCHEMA: Any
DEVICE_CONDITION_SCHEMA: Any
CONDITION_SCHEMA: vol.Schema
TRIGGER_BASE_SCHEMA: Any
TRIGGER_SCHEMA: Any
_SCRIPT_DELAY_SCHEMA: Any
_SCRIPT_WAIT_TEMPLATE_SCHEMA: Any
DEVICE_ACTION_BASE_SCHEMA: Any
DEVICE_ACTION_SCHEMA: Any
_SCRIPT_SCENE_SCHEMA: Any
_SCRIPT_REPEAT_SCHEMA: Any
_SCRIPT_CHOOSE_SCHEMA: Any
_SCRIPT_WAIT_FOR_TRIGGER_SCHEMA: Any
_SCRIPT_SET_SCHEMA: Any
SCRIPT_ACTION_DELAY: str
SCRIPT_ACTION_WAIT_TEMPLATE: str
SCRIPT_ACTION_CHECK_CONDITION: str
SCRIPT_ACTION_FIRE_EVENT: str
SCRIPT_ACTION_CALL_SERVICE: str
SCRIPT_ACTION_DEVICE_AUTOMATION: str
SCRIPT_ACTION_ACTIVATE_SCENE: str
SCRIPT_ACTION_REPEAT: str
SCRIPT_ACTION_CHOOSE: str
SCRIPT_ACTION_WAIT_FOR_TRIGGER: str
SCRIPT_ACTION_VARIABLES: str

def determine_script_action(action: dict[str, Any]) -> str: ...

ACTION_TYPE_SCHEMAS: dict[str, Callable[[Any], dict]]
currency: Any



from .const import ATTRIBUTION as ATTRIBUTION, CONF_STATION_ID as CONF_STATION_ID, SCAN_INTERVAL as SCAN_INTERVAL
from asmog import AmpioSmog
from homeassistant.components.air_quality import AirQualityEntity as AirQualityEntity
from homeassistant.const import CONF_NAME as CONF_NAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from homeassistant.util import Throttle as Throttle
from typing import Any, Final

_LOGGER: Final[Any]
PLATFORM_SCHEMA: Final[Any]

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...

class AmpioSmogQuality(AirQualityEntity):
    _ampio: Any
    _station_id: Any
    _name: Any
    def __init__(self, api: AmpioSmogMapData, station_id: str, name: Union[str, None]) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def particulate_matter_2_5(self) -> Union[str, None]: ...
    @property
    def particulate_matter_10(self) -> Union[str, None]: ...
    @property
    def attribution(self) -> str: ...
    async def async_update(self) -> None: ...

class AmpioSmogMapData:
    api: Any
    def __init__(self, api: AmpioSmog) -> None: ...
    async def async_update(self) -> None: ...



from . import auth_store as auth_store, models as models
from .const import ACCESS_TOKEN_EXPIRATION as ACCESS_TOKEN_EXPIRATION, GROUP_ID_ADMIN as GROUP_ID_ADMIN
from .mfa_modules import MultiFactorAuthModule as MultiFactorAuthModule, auth_mfa_module_from_config as auth_mfa_module_from_config
from .providers import AuthProvider as AuthProvider, LoginFlow as LoginFlow, auth_provider_from_config as auth_provider_from_config
from datetime import timedelta
from homeassistant import data_entry_flow as data_entry_flow
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from typing import Any, Dict, Optional, Tuple

EVENT_USER_ADDED: str
EVENT_USER_REMOVED: str
_MfaModuleDict = Dict[str, MultiFactorAuthModule]
_ProviderKey = Tuple[str, Optional[str]]
_ProviderDict = Dict[_ProviderKey, AuthProvider]

class InvalidAuthError(Exception): ...
class InvalidProvider(Exception): ...

async def auth_manager_from_config(hass: HomeAssistant, provider_configs: list[dict[str, Any]], module_configs: list[dict[str, Any]]) -> AuthManager: ...

class AuthManagerFlowManager(data_entry_flow.FlowManager):
    auth_manager: Any
    def __init__(self, hass: HomeAssistant, auth_manager: AuthManager) -> None: ...
    async def async_create_flow(self, handler_key: Any, *, context: Union[dict[str, Any], None] = ..., data: Union[dict[str, Any], None] = ...) -> data_entry_flow.FlowHandler: ...
    async def async_finish_flow(self, flow: data_entry_flow.FlowHandler, result: FlowResult) -> FlowResult: ...

class AuthManager:
    hass: Any
    _store: Any
    _providers: Any
    _mfa_modules: Any
    login_flow: Any
    def __init__(self, hass: HomeAssistant, store: auth_store.AuthStore, providers: _ProviderDict, mfa_modules: _MfaModuleDict) -> None: ...
    @property
    def auth_providers(self) -> list[AuthProvider]: ...
    @property
    def auth_mfa_modules(self) -> list[MultiFactorAuthModule]: ...
    def get_auth_provider(self, provider_type: str, provider_id: Union[str, None]) -> Union[AuthProvider, None]: ...
    def get_auth_providers(self, provider_type: str) -> list[AuthProvider]: ...
    def get_auth_mfa_module(self, module_id: str) -> Union[MultiFactorAuthModule, None]: ...
    async def async_get_users(self) -> list[models.User]: ...
    async def async_get_user(self, user_id: str) -> Union[models.User, None]: ...
    async def async_get_owner(self) -> Union[models.User, None]: ...
    async def async_get_group(self, group_id: str) -> Union[models.Group, None]: ...
    async def async_get_user_by_credentials(self, credentials: models.Credentials) -> Union[models.User, None]: ...
    async def async_create_system_user(self, name: str, group_ids: Union[list[str], None] = ...) -> models.User: ...
    async def async_create_user(self, name: str, group_ids: Union[list[str], None] = ...) -> models.User: ...
    async def async_get_or_create_user(self, credentials: models.Credentials) -> models.User: ...
    async def async_link_user(self, user: models.User, credentials: models.Credentials) -> None: ...
    async def async_remove_user(self, user: models.User) -> None: ...
    async def async_update_user(self, user: models.User, name: Union[str, None] = ..., is_active: Union[bool, None] = ..., group_ids: Union[list[str], None] = ...) -> None: ...
    async def async_activate_user(self, user: models.User) -> None: ...
    async def async_deactivate_user(self, user: models.User) -> None: ...
    async def async_remove_credentials(self, credentials: models.Credentials) -> None: ...
    async def async_enable_user_mfa(self, user: models.User, mfa_module_id: str, data: Any) -> None: ...
    async def async_disable_user_mfa(self, user: models.User, mfa_module_id: str) -> None: ...
    async def async_get_enabled_mfa(self, user: models.User) -> dict[str, str]: ...
    async def async_create_refresh_token(self, user: models.User, client_id: Union[str, None] = ..., client_name: Union[str, None] = ..., client_icon: Union[str, None] = ..., token_type: Union[str, None] = ..., access_token_expiration: timedelta = ..., credential: Union[models.Credentials, None] = ...) -> models.RefreshToken: ...
    async def async_get_refresh_token(self, token_id: str) -> Union[models.RefreshToken, None]: ...
    async def async_get_refresh_token_by_token(self, token: str) -> Union[models.RefreshToken, None]: ...
    async def async_remove_refresh_token(self, refresh_token: models.RefreshToken) -> None: ...
    def async_create_access_token(self, refresh_token: models.RefreshToken, remote_ip: Union[str, None] = ...) -> str: ...
    def _async_resolve_provider(self, refresh_token: models.RefreshToken) -> Union[AuthProvider, None]: ...
    def async_validate_refresh_token(self, refresh_token: models.RefreshToken, remote_ip: Union[str, None] = ...) -> None: ...
    async def async_validate_access_token(self, token: str) -> Union[models.RefreshToken, None]: ...
    def _async_get_auth_provider(self, credentials: models.Credentials) -> Union[AuthProvider, None]: ...
    async def _user_should_be_owner(self) -> bool: ...



from .const import CONF_LOCALE as CONF_LOCALE, DOMAIN as DOMAIN, PLATFORMS as PLATFORMS
from .renault_hub import RenaultHub as RenaultHub
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_PASSWORD as CONF_PASSWORD, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...



from .const import CONF_MOUNT_DIR as CONF_MOUNT_DIR, CONF_TYPE_OWSERVER as CONF_TYPE_OWSERVER, CONF_TYPE_SYSBUS as CONF_TYPE_SYSBUS
from .model import OWServerDeviceDescription as OWServerDeviceDescription
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_HOST as CONF_HOST, CONF_PORT as CONF_PORT, CONF_TYPE as CONF_TYPE
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from typing import Any

DEVICE_COUPLERS: Any

class OneWireHub:
    hass: Any
    type: Any
    pi1proxy: Any
    owproxy: Any
    devices: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    async def connect(self, host: str, port: int) -> None: ...
    async def check_mount_dir(self, mount_dir: str) -> None: ...
    async def initialize(self, config_entry: ConfigEntry) -> None: ...
    async def discover_devices(self) -> None: ...
    def _discover_devices_owserver(self, path: str = ...) -> list[OWServerDeviceDescription]: ...

class CannotConnect(HomeAssistantError): ...
class InvalidPath(HomeAssistantError): ...



from homeassistant.components.media_player import ATTR_MEDIA_CONTENT_ID as ATTR_MEDIA_CONTENT_ID, ATTR_MEDIA_CONTENT_TYPE as ATTR_MEDIA_CONTENT_TYPE, ATTR_MEDIA_SEEK_POSITION as ATTR_MEDIA_SEEK_POSITION, ATTR_MEDIA_SHUFFLE as ATTR_MEDIA_SHUFFLE, ATTR_MEDIA_VOLUME_LEVEL as ATTR_MEDIA_VOLUME_LEVEL, ATTR_MEDIA_VOLUME_MUTED as ATTR_MEDIA_VOLUME_MUTED, DOMAIN as DOMAIN, MediaPlayerEntity as MediaPlayerEntity, PLATFORM_SCHEMA as PLATFORM_SCHEMA, SERVICE_CLEAR_PLAYLIST as SERVICE_CLEAR_PLAYLIST, SERVICE_MEDIA_NEXT_TRACK as SERVICE_MEDIA_NEXT_TRACK, SERVICE_MEDIA_PAUSE as SERVICE_MEDIA_PAUSE, SERVICE_MEDIA_PLAY as SERVICE_MEDIA_PLAY, SERVICE_MEDIA_PREVIOUS_TRACK as SERVICE_MEDIA_PREVIOUS_TRACK, SERVICE_MEDIA_SEEK as SERVICE_MEDIA_SEEK, SERVICE_MEDIA_STOP as SERVICE_MEDIA_STOP, SERVICE_PLAY_MEDIA as SERVICE_PLAY_MEDIA, SERVICE_SHUFFLE_SET as SERVICE_SHUFFLE_SET, SERVICE_TURN_OFF as SERVICE_TURN_OFF, SERVICE_TURN_ON as SERVICE_TURN_ON, SERVICE_VOLUME_MUTE as SERVICE_VOLUME_MUTE, SERVICE_VOLUME_SET as SERVICE_VOLUME_SET, SUPPORT_CLEAR_PLAYLIST as SUPPORT_CLEAR_PLAYLIST, SUPPORT_NEXT_TRACK as SUPPORT_NEXT_TRACK, SUPPORT_PAUSE as SUPPORT_PAUSE, SUPPORT_PLAY as SUPPORT_PLAY, SUPPORT_PLAY_MEDIA as SUPPORT_PLAY_MEDIA, SUPPORT_PREVIOUS_TRACK as SUPPORT_PREVIOUS_TRACK, SUPPORT_SEEK as SUPPORT_SEEK, SUPPORT_SHUFFLE_SET as SUPPORT_SHUFFLE_SET, SUPPORT_STOP as SUPPORT_STOP, SUPPORT_TURN_OFF as SUPPORT_TURN_OFF, SUPPORT_TURN_ON as SUPPORT_TURN_ON, SUPPORT_VOLUME_MUTE as SUPPORT_VOLUME_MUTE, SUPPORT_VOLUME_SET as SUPPORT_VOLUME_SET, SUPPORT_VOLUME_STEP as SUPPORT_VOLUME_STEP
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, ATTR_SUPPORTED_FEATURES as ATTR_SUPPORTED_FEATURES, CONF_ENTITIES as CONF_ENTITIES, CONF_NAME as CONF_NAME, CONF_UNIQUE_ID as CONF_UNIQUE_ID, STATE_OFF as STATE_OFF, STATE_ON as STATE_ON, STATE_UNAVAILABLE as STATE_UNAVAILABLE, STATE_UNKNOWN as STATE_UNKNOWN
from homeassistant.core import HomeAssistant as HomeAssistant, State as State, callback as callback
from homeassistant.helpers.event import async_track_state_change_event as async_track_state_change_event
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType, EventType as EventType
from typing import Any, Callable

KEY_CLEAR_PLAYLIST: str
KEY_ON_OFF: str
KEY_PAUSE_PLAY_STOP: str
KEY_PLAY_MEDIA: str
KEY_SHUFFLE: str
KEY_SEEK: str
KEY_TRACKS: str
KEY_VOLUME: str
DEFAULT_NAME: str

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: Callable, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...

class MediaGroup(MediaPlayerEntity):
    _name: Any
    _state: Any
    _supported_features: int
    _attr_unique_id: Any
    _entities: Any
    _features: Any
    def __init__(self, unique_id: Union[str, None], name: str, entities: list[str]) -> None: ...
    def async_on_state_change(self, event: EventType) -> None: ...
    def async_update_supported_features(self, entity_id: str, new_state: Union[State, None]) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def state(self) -> Union[str, None]: ...
    @property
    def supported_features(self) -> int: ...
    @property
    def should_poll(self) -> bool: ...
    @property
    def device_state_attributes(self) -> dict: ...
    async def async_clear_playlist(self) -> None: ...
    async def async_media_next_track(self) -> None: ...
    async def async_media_pause(self) -> None: ...
    async def async_media_play(self) -> None: ...
    async def async_media_previous_track(self) -> None: ...
    async def async_media_seek(self, position: int) -> None: ...
    async def async_media_stop(self) -> None: ...
    async def async_mute_volume(self, mute: bool) -> None: ...
    async def async_play_media(self, media_type: str, media_id: str, **kwargs: Any) -> None: ...
    async def async_set_shuffle(self, shuffle: bool) -> None: ...
    async def async_turn_on(self) -> None: ...
    async def async_set_volume_level(self, volume: float) -> None: ...
    async def async_turn_off(self) -> None: ...
    async def async_volume_up(self) -> None: ...
    async def async_volume_down(self) -> None: ...
    def async_update_state(self) -> None: ...



from . import SimpliSafe as SimpliSafe, SimpliSafeEntity as SimpliSafeEntity
from .const import DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN, LOGGER as LOGGER
from homeassistant.components.lock import LockEntity as LockEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from simplipy.lock import Lock as Lock
from simplipy.system.v3 import SystemV3 as SystemV3
from typing import Any

ATTR_LOCK_LOW_BATTERY: str
ATTR_JAMMED: str
ATTR_PIN_PAD_LOW_BATTERY: str

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class SimpliSafeLock(SimpliSafeEntity, LockEntity):
    _lock: Any
    def __init__(self, simplisafe: SimpliSafe, system: SystemV3, lock: Lock) -> None: ...
    _attr_is_locked: bool
    async def async_lock(self, **kwargs: dict[str, Any]) -> None: ...
    async def async_unlock(self, **kwargs: dict[str, Any]) -> None: ...
    def async_update_from_rest_api(self) -> None: ...



from .const import ATTR_CID as ATTR_CID, ATTR_MAC as ATTR_MAC, ATTR_MODEL as ATTR_MODEL, CLIENTID_PREFIX as CLIENTID_PREFIX, CONF_IGNORED_SOURCES as CONF_IGNORED_SOURCES, DOMAIN as DOMAIN, NICKNAME as NICKNAME
from homeassistant import config_entries as config_entries, exceptions as exceptions
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_HOST as CONF_HOST, CONF_MAC as CONF_MAC, CONF_PIN as CONF_PIN
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from typing import Any

def host_valid(host: str) -> bool: ...

class BraviaTVConfigFlow(config_entries.ConfigFlow):
    VERSION: int
    braviarc: Any
    host: Any
    title: str
    mac: Any
    def __init__(self) -> None: ...
    async def init_device(self, pin: str) -> None: ...
    @staticmethod
    def async_get_options_flow(config_entry: ConfigEntry) -> BraviaTVOptionsFlowHandler: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_authorize(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...

class BraviaTVOptionsFlowHandler(config_entries.OptionsFlow):
    config_entry: Any
    pin: Any
    ignored_sources: Any
    source_list: Any
    def __init__(self, config_entry: ConfigEntry) -> None: ...
    async def async_step_init(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...

class CannotConnect(exceptions.HomeAssistantError): ...
class ModelNotSupported(exceptions.HomeAssistantError): ...



from homeassistant import config_entries as config_entries
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.typing import DiscoveryInfoType as DiscoveryInfoType, UNDEFINED as UNDEFINED, UndefinedType as UndefinedType
from typing import Any, Awaitable, Callable, Union

DiscoveryFunctionType = Callable[[HomeAssistant], Union[Awaitable[bool], bool]]
_LOGGER: Any

class DiscoveryFlowHandler(config_entries.ConfigFlow):
    VERSION: int
    _domain: Any
    _title: Any
    _discovery_function: Any
    def __init__(self, domain: str, title: str, discovery_function: DiscoveryFunctionType) -> None: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_confirm(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_discovery(self, discovery_info: DiscoveryInfoType) -> FlowResult: ...
    async_step_zeroconf: Any
    async_step_ssdp: Any
    async_step_mqtt: Any
    async_step_homekit: Any
    async_step_dhcp: Any
    async def async_step_import(self, _: Union[dict[str, Any], None]) -> FlowResult: ...

def register_discovery_flow(domain: str, title: str, discovery_function: DiscoveryFunctionType, connection_class: Union[str, UndefinedType] = ...) -> None: ...

class WebhookFlowHandler(config_entries.ConfigFlow):
    VERSION: int
    _domain: Any
    _title: Any
    _description_placeholder: Any
    _allow_multiple: Any
    def __init__(self, domain: str, title: str, description_placeholder: dict, allow_multiple: bool) -> None: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...

def register_webhook_flow(domain: str, title: str, description_placeholder: dict, allow_multiple: bool = ...) -> None: ...
async def webhook_async_remove_entry(hass: HomeAssistant, entry: config_entries.ConfigEntry) -> None: ...
